package com.commtrain.PMS.application;

import java.lang.reflect.*;

import org.apache.log4j.Logger;
import org.jpos.iso.*;

import com.commtrain.PMS.transaction.*;

public class Application implements ISORequestListener{
	
	private static Logger logger = Logger.getLogger(Application.class);

    protected ISOPackager packager;//null means don't change it
    protected ISOFilter filter;//null means don't change it

    public static final int HYPERCOM_ID=1;
    public static final int WEBSHOP_ID=2;
    public static final int ORANGE_ID=3;

    public static final int HYPERCOM_NII=99;

	public static final int NII_PREPAY_TEST = 10;
	public static final int NII_PREPAY_PRODUCTION_A = 11;
	public static final int NII_PREPAY_PRODUCTION_B = 12;

	public static final int NII_WEBSHOP_TEST = 20;
	public static final int NII_WEBSHOP_PRODUCTION_A = 21;
	public static final int NII_WEBSHOP_PRODUCTION_B = 22;

	public static final int NII_PAYMENT_TEST = 30;
	public static final int NII_PAYMENT_PRODUCTION_A = 31;
	public static final int NII_PAYMENT_PRODUCTION_B = 32;


    public static final int WEBSHOP_NII=20;
//    public static final int ORANGE_NII=10;
//    public static final int ORANGE_PAYMENT_NII=30;
    
    private static final String APPLICATION_CLASSES[] = { "",
    			"com.commtrain.PMS.application.HypercomApplication", 
    			"com.commtrain.PMS.application.WebShopApplication",
    			"com.commtrain.PMS.application.OrangeApplication"};
    

    /** application ID */
    protected int ID;
    /**
     * Constructor for Application.
     */
    public Application() {
    }

    public Application(int id) {
    	ID = id;
    }

    public boolean process(ISOSource source, ISOMsg m) {
        ISOMsg response = null;

        try {
        	ISOMsg decodedMsg = decodeRequestMsg(m);
        	ISOMsg transactionResponseMsg = processTransaction(decodedMsg);
        	if(transactionResponseMsg!=null)
	        	response = encodeResponseMsg(transactionResponseMsg);
        } catch (ISOException e) {
       		logger.error("Stack Trace", e);
        }


    	if(response==null)
    		response = Transaction.createErrorMessage(m, "","", Transaction.RESPONSE_CODE_INVALID_TRANSACTION);
    	response.setHeader(m.getHeader());

        
        try {
            source.send(response);
        } catch (Exception e) {
       		logger.error("Stack Trace", e);
            return false;
        }
        
		//free resources
/*		transaction = null;
		response = null;
        System.gc();*/
        
        return true;
    }

    protected ISOMsg decodeRequestMsg(ISOMsg m) throws ISOException {

    	ISOMsg retMsg=null;

        if (packager != null) {
            byte[] binary = m.pack();
            retMsg = new ISOMsg();
            retMsg.setPackager(packager);
            retMsg.unpack(binary);
            retMsg.setHeader(m.getHeader());
            retMsg.setDirection(m.getDirection());
        } 
        if(filter!=null){
        	retMsg = filter.filter(null, m , null);
        	
        }else
        	retMsg = m;

        return retMsg;
    }

    protected ISOMsg encodeResponseMsg(ISOMsg m) throws ISOException {

    	ISOMsg retMsg=null;

/*        if (packager != null) {
            byte[] binary = m.pack();
            retMsg = new ISOMsg();
            retMsg.setPackager(packager);
            retMsg.unpack(binary);
            retMsg.setHeader(m.getHeader());
            retMsg.setDirection(m.getDirection());
        } */
        if(filter!=null && m!=null){
        	m.setDirection(ISOMsg.OUTGOING);
        	retMsg = filter.filter(null, m , null);
        	
        }else
        	retMsg = m;

        return retMsg;
    }


    
    public ISOMsg processTransaction(ISOMsg m) throws ISOException {
        Transaction transaction = Transaction.getInstance(this, m);
        ISOMsg response = null;

        try {
            if (transaction != null)
                response = transaction.process();
        } catch (Throwable e) {
            logger.error("Stack Trace", e);
        }
        return response;
    }
    

    /**
     * Returns the application ID.
     * @return int
     */
    public int getID() {
        return ID;
    }
    
    public static Application getInstance(ISOMsg isoRequest){
    	Application application=null;
/*    	byte[] headerBytes = isoRequest.getHeader();
    	if(headerBytes==null)
    		return app;
    		
    	IENHeader header = new IENHeader();
    	header.unpack(headerBytes);
        String clientType = Byte.toString(header.getId());*/
		String niiStr = TransactionUtils.getString(isoRequest, Transaction.NET_INTERNATIONAL_IDENTIFIEER);
		if(niiStr==null){
			logger.warn("NII not set in request. Ignoring the request!");
			return application;
		}
		
		int nii =-1;
		try{
			nii = Integer.parseInt(niiStr);
		}catch(NumberFormatException e){
		}
		
		int appID=-1;
		switch (nii) {
            case HYPERCOM_NII:
                appID = HYPERCOM_ID;
                break;

			case NII_WEBSHOP_TEST:
			case NII_WEBSHOP_PRODUCTION_A:
			case NII_WEBSHOP_PRODUCTION_B:
                appID = WEBSHOP_ID;
                break;
			
			case NII_PREPAY_TEST:
			case NII_PREPAY_PRODUCTION_A:
			case NII_PREPAY_PRODUCTION_B:
                appID = ORANGE_ID;
                break;
			
			case NII_PAYMENT_TEST:
			case NII_PAYMENT_PRODUCTION_A:
			case NII_PAYMENT_PRODUCTION_B:
                appID = HYPERCOM_ID;
                break;
        }


        try {
        	String applicationClassName = APPLICATION_CLASSES[appID];
            Class classInst = Class.forName(applicationClassName);
            Class[] parameters = {};
            Constructor constructor = classInst.getConstructor(parameters);
            application = (Application) constructor.newInstance(new Object[] {});
        } catch (NoSuchMethodException e) {
        } catch (InvocationTargetException e) {
        } catch (InstantiationException e) {
        } catch (IllegalAccessException e) {
        } catch (ClassNotFoundException e) {
        } catch (ArrayIndexOutOfBoundsException e) {
        }


    	return application;
    }

}