package com.commtrain.PMS.db;

import java.sql.Connection;
import java.sql.SQLException;

import oracle.jbo.server.ConnectionPoolManager;
import oracle.jbo.server.ConnectionPoolManagerFactory;

import org.apache.log4j.BasicConfigurator;

/**
 * Use it to get and release connections to the database.
 * 
 */
public class Connector {
    private static org.apache.log4j.Logger logger = org.apache.log4j.Logger.getLogger(Connector.class);

    private static String jdbcUser = "cdsman";
    private static String jdbcPassword = "manager";
    private static String jdbcURL = "jdbc:oracle:thin:@cca-dev5.commtrain-dev.com:1521:oradb";       //"jdbc:oracle:oci8:@oradb";
    private static String jdbcDriver = "oracle.jdbc.driver.OracleDriver";
	private static String schemaName = "CDSMAN";

    protected static String __connectionPoolKey;
    protected static ConnectionPoolManager __connectionPoolManager;
    protected static long connectionsGot=0;
    protected static long connectionsReleased=0;

    /**
     * Method getConnection.
     * @return Connection
     */
    public static Connection getConnection() throws SQLException {

        if (__connectionPoolKey == null) {
        	
        	//Load Oracle Driver
        	try{
	            Class.forName(jdbcDriver).newInstance();
        	}catch(Exception e){
        		logger.error("Driver " + jdbcDriver + " could not be loaded!");
        		return null;
        	}
        	
        	System.setProperty("jbo.ConnectionPoolManager","oracle.jbo.server.ConnectionPoolManagerImpl");
//        	System.setProperty("jbo.poolmaxinactiveage","10000");//10 sec
//        	System.setProperty("jbo.poolmonitorsleepinterval","2000");//2 sec WARNING
        	System.setProperty("jbo.poolminavailablesize","0");//connection pool's initial size

        	System.setProperty("jbo.debugoutput", "com.commtrain.PMS.db.Log4jDiagnosticsImpl");

        	
	        // Get a connection pool manager instance
	        // from the connection pool manager factory
	        __connectionPoolManager = ConnectionPoolManagerFactory.getConnectionPoolManager();

            // Generate a unique signature for the pool.  Assume
            // that the JDBC URL, JDBC user, and JDBC password were
            // all specified as separate properties.  Note that
            // the connection pool key is stored as member variable
            // for later use.
            __connectionPoolKey = __connectionPoolManager.generatePoolKey(jdbcURL, jdbcUser, jdbcPassword);
        }

        // Get a connection from the pool for the key that
        // was generated above.  The URL, user, and password
        // passed in case a connection is not available and a
        // new connection must be generated.
        Connection conn = __connectionPoolManager.getConnection(__connectionPoolKey, jdbcURL, null, jdbcUser, jdbcPassword);
        if(conn==null)
        	throw new SQLException("Connector couldn't get a connection to db!");
        connectionsGot++;
        return conn;
    }

    /**
     * Method releaseConnection.
     */
    public static void releaseConnection(Connection conn) {
        // Return the connection to the connection pool.  The target
        // connection pool is identified with the connection pool key
        // that was generated while checking out a connection above.
        // The jdbcConn
        
        
        if(__connectionPoolManager!=null&&conn!=null){
	        __connectionPoolManager.returnConnection(__connectionPoolKey, conn);
	        connectionsReleased++;
        }
    }


    public static void setJDBCParams(String driver, String connurl, String username, String password){
   		jdbcDriver = driver;
		jdbcURL = connurl;
    	jdbcUser = username;
    	jdbcPassword = password;
    }
	public static void setSchemaName(String name){
			schemaName=name;
	}
	public static String getSchemaName(){
			return schemaName;
	}	

    public static void printStatus() {
        logger.info("Connections got:" + connectionsGot
                  + " released:" + connectionsReleased);
    }

    public static void main(String argv[]) {
        BasicConfigurator.configure();
        Connection conn = null;
        try {
            conn = getConnection();
            logger.info(conn);

        } catch (SQLException e) {
        } finally {
            releaseConnection(null);
        }
    }

}