package com.commtrain.PMS.db;

import java.sql.*;
import java.util.Date;

import java.math.*;
import org.jpos.iso.*;

/**
 *
 * Database procedures and functions.
 * Requirements:
 * 	1) all methods are static
 * 	2) no class fields which are not final
 * 	3) all parameter types and return values for the public methods are primitive types which are easy to bind to SQL Types.
 */
public class ProcsFuncs {
	
    private static org.apache.log4j.Logger logger = org.apache.log4j.Logger.getLogger(ProcsFuncs.class);

	public static class CDSCallResult{
		private BigDecimal cdsID;
		private BigDecimal newBalance;
		private BigDecimal reservationAmount;
		private  int resCode;
		CDSCallResult(BigDecimal cdsID, BigDecimal newBalance, BigDecimal reservationAmount, int resCode){
			this.cdsID = cdsID;
			this.newBalance = newBalance;
			this.reservationAmount = reservationAmount;
			this.resCode = resCode;
		}
        /**
         * Returns the cdsID.
         * @return BigDecimal
         */
        public BigDecimal getCdsID() {
            return cdsID;
        }

        /**
         * Returns the newBalance.
         * @return BigDecimal
         */
        public BigDecimal getNewBalance() {
            return newBalance;
        }

        /**
         * Returns the resCode.
         * @return int
         */
        public int getResCode() {
            return resCode;
        }

        /**
         * Returns the reservationAmount.
         * @return BigDecimal
         */
        public BigDecimal getReservationAmount() {
            return reservationAmount;
        }

	}

    /**
     * Method getConnection.
     * @return Connection
     */
    private static Connection getConnection() throws SQLException {
        return Connector.getConnection();
    }

    /**
     * Method releaseConnection.
     */
    private static void releaseConnection(Connection conn) {
        Connector.releaseConnection(conn);
    }

    protected static int getCardType(String track2Data) {
        int crdType = 0;
        Connection conn = null;
        CallableStatement cstmt = null;
        try {
            conn = Connector.getConnection();
            cstmt = conn.prepareCall("{? = call CARD_TYPE(?)}");

            cstmt.registerOutParameter(1, Types.INTEGER);
            cstmt.setString(2, track2Data);
            cstmt.execute();

            crdType = cstmt.getInt(1);
        } catch (Exception e) {
            logger.error("Stack Trace", e);
        } finally {
            if (cstmt != null) {
                try {
                    cstmt.close();
                } catch (SQLException e) {
                }
            }
            if (conn != null)
                releaseConnection(conn);
        }
        return crdType;
    }

    public static int getCarSeqNo(String trmID, int acqID) {
        int seqNo = -1;
        Connection conn = null;
        CallableStatement cstmt = null;
        try {
            conn = Connector.getConnection();
            cstmt = conn.prepareCall("{? = call NEXT_SEQ_CAR_RQ(?, ?)}");

            cstmt.registerOutParameter(1, Types.INTEGER);
            cstmt.setString(2, trmID);
            cstmt.setInt(3, acqID);
            cstmt.execute();

            seqNo = cstmt.getInt(1);
        } catch (Exception e) {
            logger.error("Stack Trace", e);
        } finally {
            if (cstmt != null) {
                try {
                    cstmt.close();
                } catch (SQLException e) {
                }
            }
            if (conn != null)
                releaseConnection(conn);
        }
        return seqNo;
    }

    public static BigDecimal addFakeCDSRecord(ISOMsg msg) {
        Connection conn = null;
        CallableStatement cstmt = null;

        BigDecimal cdsID = null;
/*        String trmID = msg.getString(Transaction.TERMINAL_IDENTIFICATION);
        String mti = msg.getString(Transaction.MESSAGE_TYPE_INDICATOR);
        String processingCode = msg.getString(Transaction.PROCESSING_CODE);
        String sysTraceAuditNo = msg.getString(Transaction.SYSTEM_TRACE_AUDIT_NUM);

        int resCode = 0;
*/        try {

            conn = Connector.getConnection();
            //	Register the transaction in CDS

            cstmt = conn.prepareCall("{call INS_CDS_FAKE (?, ?)}");

        } catch (Exception e) {
            logger.error("Stack Trace", e);
        } finally {
            if (cstmt != null) {
                try {
                    cstmt.close();
                } catch (SQLException e) {
                }
            }
            if (conn != null)
                releaseConnection(conn);
        }

        return cdsID;
    }


    public static void updateStatus(Connection conn, BigDecimal cdsID, int status) throws SQLException {
        CallableStatement cstmt = null;
        try {
            cstmt = conn.prepareCall("{call UPDATE_CDS_STATUS(?, ?)}");
            cstmt.setBigDecimal(1, cdsID);
            cstmt.setInt(2, status);
            cstmt.execute();
        } finally {
            if (cstmt != null) {
                try {
                    cstmt.close();
                } catch (SQLException e) {
                }
            }
        }
    }

    public static void updateStatus(BigDecimal cdsID, int status) {
        Connection conn = null;
        try {
            conn = Connector.getConnection();
            updateStatus(conn, cdsID, status);
        } catch (Exception e) {
            logger.error("Stack Trace", e);
        } finally {
            if (conn != null)
                releaseConnection(conn);
        }
    }

   public static int getStatus(BigDecimal cdsID) {
        Connection conn = null;
        CallableStatement cstmt = null;
        int status = -1;
        try {
            conn = Connector.getConnection();
            cstmt = conn.prepareCall("{? = call CDS_STATUS(?)}");
            cstmt.registerOutParameter(1, Types.INTEGER);
            cstmt.setBigDecimal(2, cdsID);
            cstmt.execute();
			status = cstmt.getInt(1);
        } catch (Exception e) {
            logger.error("Stack Trace", e);
        } finally {
            if (cstmt != null) {
                try {
                    cstmt.close();
                } catch (SQLException e) {
                }
            }
            if (conn != null)
                releaseConnection(conn);
        }
        return status;
    }

    /**
     * Update CDS RP field in the database.
     * @param Connection conn SQL Connection to be used
     * @param BigDecimal cdsID the id of the CDS record
     * @param Integer responseIndicator could be null
     * @param String responseCode
     * @param String approvalCode
     * @param Integer floorLimit could be null
     * @param Integer newStatus could be null
     */
    public static void updCDS_RP(
    	Connection conn,
    	BigDecimal cdsID,
    	Integer responseIndicator, 
    	String carResponseCode, 
    	String approvalCode, 
    	Integer floorLimit,
    	int newStatus) throws SQLException {

        CallableStatement cstmt = null;
       
        try {
            cstmt = conn.prepareCall("{call UPDATE_CDS_RP(?, ?, ?, ?, ?, ?)}");
            cstmt.setBigDecimal(1, cdsID);
            if(responseIndicator!=null)
	            cstmt.setBigDecimal(2, new BigDecimal(responseIndicator.intValue()));
            cstmt.setString(3, carResponseCode);
            cstmt.setString(4, approvalCode);
            if(floorLimit!=null)
	            cstmt.setBigDecimal(5, new BigDecimal(floorLimit.intValue()));
            cstmt.setBigDecimal(6, new BigDecimal(newStatus));
            cstmt.execute();
        } finally {
            if (cstmt != null) {
                try {
                    cstmt.close();
                } catch (SQLException e) {
                }
            }
        }

    }


    /**
     * Update CDS RP field in the database.
     * @param BigDecimal cdsID the id of the CDS record
     * @param Integer responseIndicator could be null
     * @param String responseCode
     * @param String approvalCode
     * @param Integer floorLimit could be null
     * @param Integer newStatus could be null
     */
    public static void updCDS_RP(
    	BigDecimal cdsID,
    	Integer responseIndicator, 
    	String carResponseCode, 
    	String approvalCode, 
    	Integer floorLimit,
    	int newStatus) {

        Connection conn = null;
       
        try {
            conn = Connector.getConnection();
            updCDS_RP(conn, cdsID, responseIndicator, carResponseCode, approvalCode, floorLimit, newStatus);
        } catch (Exception e) {
            logger.error("Stack Trace", e);
        } finally {
            if (conn != null)
                releaseConnection(conn);
        }

    }


    /**
     * Call INS_CS.
     * @param Connection conn SQLConnection
     * @param String trmID terminal ID
     * @param int transactionCode
	 * @param BigDecimal amount
	 * @param BigDecimal cdsID
     * @return int resCode the result code
     */
    public static int addCSRecord(
    	Connection conn,
    	String trmID, 
    	int transactionCode,  
    	int status,  	
    	BigDecimal amount,
    	BigDecimal cdsID) throws SQLException {

		int resCode = -1;
        CallableStatement cstmt = null;
        
        try {
            cstmt = conn.prepareCall("{call INS_CS(?, ?, ?, ?, ?, ?)}");
            cstmt.setString(1, trmID);
            cstmt.setInt(2, transactionCode);
            cstmt.setInt(3, status);
            cstmt.setBigDecimal(4, amount);
            cstmt.setBigDecimal(5, cdsID);
            cstmt.registerOutParameter(6, Types.NUMERIC);
            cstmt.execute();
            resCode = cstmt.getInt(6);
        } finally {
            if (cstmt != null) {
                try {
                    cstmt.close();
                } catch (SQLException e) {
                    logger.warn("Stack Trace", e);
                }
            }
        }
		return resCode;
    }


    /**
     * Call INS_CS.
     * @param String trmID terminal ID
     * @param int transactionCode
	 * @param BigDecimal amount
	 * @param BigDecimal cdsID
     * @return int resCode the result code
     * @deprecated
     */
    public static int addCSRecord(
    	String trmID, 
    	int transactionCode,
    	int status, 
    	BigDecimal amount,
    	BigDecimal cdsID) {

		int resCode = -1;
        Connection conn = null;
        try {
            conn = Connector.getConnection();
            resCode = addCSRecord(conn, trmID, transactionCode, status,  amount, cdsID);
        } catch (Exception e) {
            logger.error("Stack Trace", e);
        } finally {
            if (conn != null)
                releaseConnection(conn);
        }
		return resCode;
    }

	/**
     * Calls voidProceeded.
     * @param String trmID terminal ID
	 * @param String invoiceNo Invoice number from iso msg
     * @param int transactionCode
     * @return int resCode the result code
     */
	public static int voidProceeded(
	   	String trmID, 
    	String invoiceNo,
    	int transactionCode)throws SQLException{

	int resCode = -1;
        Connection conn = null;
        try {
            conn = Connector.getConnection();
            resCode = voidProceeded(conn, trmID, invoiceNo, transactionCode);
        } catch (Exception e) {
            logger.error("Stack Trace", e);
        } finally {
            if (conn != null)
                releaseConnection(conn);
        }
		return resCode;
	}
	
	
	/**
     * Calls VOID_PROCEEDED
     * @param Connection conn  Connection to the Data Base
     * @param String trmID terminal ID
	 * @param String invoiceNo Invoice number from iso msg
     * @param int transactionCode
     * @return int resCode the result code
     */
	public static int voidProceeded(
		Connection conn,
	   	String trmID, 
    	String invoiceNo,
    	int transactionCode)throws SQLException{


		int resCode = -1;
        CallableStatement cstmt = null;
        try {
            cstmt = conn.prepareCall("{? = call VOID_PROCEEDED(?, ?, ?)}");
            cstmt.registerOutParameter(1, Types.INTEGER);
            cstmt.setString(2, trmID);
            cstmt.setString(3, invoiceNo);
            cstmt.setInt(4, transactionCode);
            cstmt.execute();
            resCode = cstmt.getInt(1);
        } finally {
            if (cstmt != null) {
                try {
                    cstmt.close();
                } catch (SQLException e) {
                    logger.warn("Stack Trace", e);
                }
            }
        }
		return resCode;	 
	}
	

    /**
     * Call VOID_CS.
     * @param conn SQLConnection
     * @param trmID terminal ID
	 * @param invoiceNo Invoice number from iso msg
     * @param transactionCode
	 * @param amount Amount from iso msg
     * @return the status of the original transaction
     *          or 0 if the original transaction was not found
     */
    public static int voidCSRecord(
    	Connection conn,
    	String trmID, 
    	String invoiceNo,
    	int transactionCode,
    	BigDecimal amount) throws SQLException {

		int status;
        CallableStatement cstmt = null;
        try {
            cstmt = conn.prepareCall("{call VOID_CS(?, ?, ?, ?, ?)}");
            cstmt.setString(1, trmID);
            cstmt.setString(2, invoiceNo);
            cstmt.setInt(3, transactionCode);
            cstmt.setBigDecimal(4, amount);
            cstmt.registerOutParameter(5, Types.INTEGER);
            cstmt.execute();
            status = cstmt.getInt(5);
        } finally {
            if (cstmt != null) {
                try {
                    cstmt.close();
                } catch (SQLException e) {
                    logger.warn("Stack Trace", e);
                }
            }
        }
		return status;
    }

    /**
     * Call VOID_CS.
     * @param trmID terminal ID
	 * @param invoiceNo Invoice number from iso msg
     * @param transactionCode
	 * @param amount Amount from iso msg
     * @return the status of the original transaction
     *          or 0 if the original transaction was not found
     */
    public static int voidCSRecord(
    	String trmID, 
    	String invoiceNo,
    	int transactionCode,
    	BigDecimal amount) {

		int status = 0;
        Connection conn = null;
        try {
            conn = Connector.getConnection();
            status = voidCSRecord(conn, trmID, invoiceNo, transactionCode, amount);
        } catch (Exception e) {
            logger.error("Stack Trace", e);
        } finally {
            if (conn != null)
                releaseConnection(conn);
        }
		return status;
    }

    /**
     * Call PROCESS_WEBBATCH.
     * @param Connection conn
     * @param String trmID terminal ID
	 * @param Date startTransactionDate used to calculate batchDate in database
     * @return int resCode the result code
     */
    public static int processWebBatch(
    	Connection conn,
    	String trmID, 
    	Date startTransactionDate) {

		int resCode = -1;
        CallableStatement cstmt = null;
        try {
            cstmt = conn.prepareCall("{call PROCESS_WEBBATCH(?, ?, ?)}");
            cstmt.setString(1, trmID);
            cstmt.setTimestamp(2, new Timestamp(startTransactionDate.getTime())); 
            cstmt.registerOutParameter(3, Types.NUMERIC);
            cstmt.execute();
            resCode = cstmt.getInt(3);
        } catch (Exception e) {
            logger.error("Stack Trace", e);
        } finally {
            if (cstmt != null) {
                try {
                    cstmt.close();
                } catch (SQLException e) {
                    logger.warn("Stack Trace", e);
                }
            }
        }
		return resCode;
    }

    /**
     * Call PROCESS_WEBBATCH_ALL.
     * @param Connection conn
     * @param String merchantID Merchant ID
	 * @param Date startTransactionDate used to calculate batchDate in database
     * @return int resCode the result code
     */
    public static int processWebBatchAll(
    	Connection conn,
    	String merchantID, 
    	Date startTransactionDate) {

		int resCode = -1;
        CallableStatement cstmt = null;
        try {
            cstmt = conn.prepareCall("{call PROCESS_WEBBATCH_ALL(?, ?, ?)}");
            cstmt.setString(1, merchantID);
            cstmt.setTimestamp(2, new Timestamp(startTransactionDate.getTime())); 
            cstmt.registerOutParameter(3, Types.NUMERIC);
            cstmt.execute();
            resCode = cstmt.getInt(3);
        } catch (Exception e) {
            logger.error("Stack Trace", e);
        } finally {
            if (cstmt != null) {
                try {
                    cstmt.close();
                } catch (SQLException e) {
                    logger.warn("Stack Trace", e);
                }
            }
        }
		return resCode;
    }
}
