package com.commtrain.PMS.db;
import java.sql.*;
import java.math.*;

import org.apache.log4j.BasicConfigurator;


public class TransactionContext {
	
    private static org.apache.log4j.Logger logger = org.apache.log4j.Logger.getLogger(TransactionContext.class);
    
	private Connection conn = null;
	private CallableStatement cstmt = null;
	private int resCode = 0;
	private String trmID;
	private boolean isLoaded = false;

// procedure prameters
	private String trmCompany;
	private String trmCity;
	private String trmManufacturer;
	private String trmModel;
    private String trmACQID;
    private String trmACQIdMOTO;
    private String trmOptions;
	private String vUNumber;
    private String vUNumberMOTO;
	private BigDecimal acqID;
	private String acqHelpDeskNo;
	private String acqISDNNo;
	private String acqX25No;
	private String acqIPHost;
	private String acqIPPort;
	private int acqNII;
	private int acqCommMethod;
	private BigDecimal acqSubmitterID;
	private BigDecimal acqManufacturerID;
	private int cardType;
   

//	Parameter positions
//	Input Params
	static final private int CARD_DETAILS_POS = 1;
	static final private int TRM_ID_POS = 2;
//	Output Params
	static final private int TRM_COMPANY_POS = 3;
	static final private int TRM_CITY_POS = 4;
	static final private int TRM_MANUFACTURER_POS = 5;
	static final private int TRM_MODEL_POS = 6;
	static final private int TRM_ACQ_ID_POS = 7;
    static final private int TRM_ACQ_ID_POS_MOTO = 8;
    static final private int TRM_OPTIONS = 9;
	static final private int CTR_VU_NUMBER_POS = 10;
    static final private int CTR_VU_NUMBER_POS_MOTO = 11;
	static final private int ACQ_ID_POS = 12;
	static final private int ACQ_HELPDESK_NO_POS = 13;
	static final private int ACQ_NII_POS = 14;
	static final private int ACQ_COMM_METHOD_POS = 15;
	static final private int ACQ_ISDN_NO_POS = 16;
	static final private int ACQ_X25_NO_POS = 17;
	static final private int ACQ_HOST_POS = 18;
	static final private int ACQ_PORT_POS = 19;
	static final private int ACQ_SUBMITER_ID_POS = 20;
	static final private int ACQ_MANUFACTURER_ID_POS = 21;
    static final private int CARD_TYPE_POS = 22;
    static final private int LICENSE_NO_POS = 23;
    static final private int RES_CODE_POS = 24;

//	static final private int SEQ_NO_POS = 99;
//procedure parameters end	

	public static final int COMM_METHOD_IEN = 0;		// IEN Routing table
	public static final int COMM_METHOD_IP = 1;		// Direct IP communication
	public static final int COMM_METHOD_X25 = 2;		// Direct X25 communication
	public static final int COMM_METHOD_ISDN = 3;	// Direct ISDN communication
	public static final int COMM_METHOD_MQ = 4;		// MQ Series message queues
	public static final int COMM_METHOD_UNKNOWN = -1;// Unknown



	protected boolean releaseConn = false;

	public TransactionContext(String cardDetails , String trmID) throws SQLException {
		this(null, cardDetails, trmID);
		releaseConn= true;
	}

	public TransactionContext(Connection conn, String cardDetails, String trmID) throws SQLException {
		this.trmID = trmID;
		this.conn = conn;
		if(conn==null){
			releaseConn = true;
			this.conn = Connector.getConnection();
		}else{
			releaseConn = false;
			this.conn = conn;
		}
		callGetTransactionContext(cardDetails);
	}
	
	public boolean isLoaded() {
		return isLoaded;
	}

	public int getCardType() {
		return cardType;
	}

    public String getLocaleDescription() {
        return trmOptions != null ? trmOptions : "";
    }
	
	public String getTrmID() {
		return trmID;
	}
	
	private void readFields() throws SQLException {
			trmCompany = cstmt.getString(TRM_COMPANY_POS);
			trmCity = cstmt.getString(TRM_CITY_POS);
			trmManufacturer = cstmt.getString(TRM_MANUFACTURER_POS);
			trmModel = cstmt.getString(TRM_MODEL_POS);
			trmACQID = cstmt.getString(TRM_ACQ_ID_POS);
            trmACQIdMOTO = cstmt.getString(TRM_ACQ_ID_POS_MOTO);
            trmOptions = cstmt.getString(TRM_OPTIONS);
			vUNumber = cstmt.getString(CTR_VU_NUMBER_POS);
            vUNumberMOTO = cstmt.getString(CTR_VU_NUMBER_POS_MOTO);
			acqID = cstmt.getBigDecimal(ACQ_ID_POS);
			acqHelpDeskNo = cstmt.getString(ACQ_HELPDESK_NO_POS);
			acqISDNNo = cstmt.getString(ACQ_ISDN_NO_POS);
			acqX25No = cstmt.getString(ACQ_X25_NO_POS);
			acqIPHost = cstmt.getString(ACQ_HOST_POS);
			acqIPPort = cstmt.getString(ACQ_PORT_POS);
			acqCommMethod = cstmt.getInt(ACQ_COMM_METHOD_POS);
			acqNII = cstmt.getInt(ACQ_NII_POS);
			acqSubmitterID = cstmt.getBigDecimal(ACQ_SUBMITER_ID_POS);
			acqManufacturerID = cstmt.getBigDecimal(ACQ_MANUFACTURER_ID_POS);
			resCode = cstmt.getBigDecimal(RES_CODE_POS).intValue();
			cardType = cstmt.getBigDecimal(CARD_TYPE_POS).intValue();
            byte[] licenseNo = cstmt.getBytes(LICENSE_NO_POS);
            
	}
    
	public String getTrmCompany(){
		return trmCompany;
	}
    
	public String getTrmCity(){
		return trmCity ;
	}
	
	public String getTrmManufacturer(){
			return trmManufacturer;		
	}
	
	public String getTrmModel(){
		return trmModel;		
	}

	public String getTrmAcqID(){
		return trmACQID;		
	}

    public String getTrmAcqIdMOTO(){
        return trmACQIdMOTO;        
    }

    public String getTrmOptions(){
        return trmOptions;        
    }

	public String getVUNumber(){
		return vUNumber;
	}

    public String getVUNumberMOTO(){
        return vUNumberMOTO;
    }

	public BigDecimal getAcqID(){
		return acqID;
	}

	public String getAcqHelpdeskNo(){
			return acqHelpDeskNo;
	}

	public String getAcqISDNNo(){
			return acqISDNNo;
	}

	public String getAcqX25No(){
		return acqX25No;
	}

	public BigDecimal getAcqSubmiterID(){
			return acqSubmitterID;
	}
    
	public  BigDecimal getAcqManufacturerID(){
			return acqManufacturerID;
	}
    
	public int getRescode(){
			return resCode;
	}	
    
	public int getAcqNII(){
		return acqID.intValue();
	}

	public int getAcqCommMethod(){
		switch (acqCommMethod) {
			case COMM_METHOD_IEN:
			case COMM_METHOD_IP:
			case COMM_METHOD_ISDN:
			case COMM_METHOD_X25:
			case COMM_METHOD_MQ:
				break;
			default:
				this.acqCommMethod = COMM_METHOD_UNKNOWN;
				break;
		}
		return acqCommMethod;
	}

    public String getAcqIPHost() {
        return acqIPHost;
//        return "53.250.138.120";
    }

    public int getAcqIPPort() {
        try {
            return Integer.parseInt(acqIPPort);
//            return 44321;
        } catch (Exception e) {
            return 44321;
        }
    }

   

	protected void callGetTransactionContext(String track2Data) throws SQLException  {
		if (conn == null || trmID == null)
			return;
			
		try{
			cstmt = conn.prepareCall(
	           "{call GET_TRANSACTION_CONTEXT (?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?)}"
	           );
	        // Input Parameters
	        cstmt.setString(CARD_DETAILS_POS, track2Data);	                    //crd_type_id IN number
	        cstmt.setString(TRM_ID_POS, trmID);								    //trm_id IN varchar2
	        // Output Parameters
	        cstmt.registerOutParameter(TRM_COMPANY_POS,  Types.VARCHAR);	    //trm_company OUT varchar2
	        cstmt.registerOutParameter(TRM_CITY_POS,  Types.VARCHAR);		    //trm_city OUT varchar2
	        cstmt.registerOutParameter(TRM_MANUFACTURER_POS,  Types.VARCHAR);   //trm_manufacturer OUT varchar2
	        cstmt.registerOutParameter(TRM_MODEL_POS,  Types.VARCHAR);		    //trm_model OUT varchar2
	        cstmt.registerOutParameter(TRM_ACQ_ID_POS,  Types.VARCHAR);		    //trm_acq_id OUT varchar2
            cstmt.registerOutParameter(TRM_ACQ_ID_POS_MOTO,  Types.VARCHAR);    //trm_acq_id_moto OUT varchar2
            cstmt.registerOutParameter(TRM_OPTIONS,  Types.VARCHAR);            //trm_options OUT varchar2
	        cstmt.registerOutParameter(CTR_VU_NUMBER_POS,  Types.VARCHAR);	    //ctr_vu_number OUT varchar2
            cstmt.registerOutParameter(CTR_VU_NUMBER_POS_MOTO,  Types.VARCHAR); //ctr_vu_number_moto OUT varchar2
			cstmt.registerOutParameter(ACQ_ID_POS,  Types.NUMERIC);			    //acq_id OUT number
			cstmt.registerOutParameter(ACQ_HELPDESK_NO_POS, Types.VARCHAR);	    //acq_helpdesk_no OUT varchar2

			cstmt.registerOutParameter(ACQ_NII_POS, Types.NUMERIC);
			cstmt.registerOutParameter(ACQ_COMM_METHOD_POS, Types.NUMERIC);

			cstmt.registerOutParameter(ACQ_ISDN_NO_POS, Types.VARCHAR);		    //acq_isdn_no OUT varchar2
			cstmt.registerOutParameter(ACQ_X25_NO_POS, Types.VARCHAR);		    //acq_x25_no OUT varchar2
			cstmt.registerOutParameter(ACQ_HOST_POS, Types.VARCHAR);		        //acq_host OUT varchar2
			cstmt.registerOutParameter(ACQ_PORT_POS, Types.VARCHAR);		        //acq_port OUT varchar2
			cstmt.registerOutParameter(ACQ_SUBMITER_ID_POS, Types.NUMERIC);	    //acq_submiter_id OUT number
			cstmt.registerOutParameter(ACQ_MANUFACTURER_ID_POS, Types.NUMERIC);	//acq_manufacturer_id OUT number
			cstmt.registerOutParameter(CARD_TYPE_POS, Types.NUMERIC);		    //card_type_id OUT number
            cstmt.registerOutParameter(LICENSE_NO_POS, Types.VARBINARY);            //licenseNo OUT RAW
			cstmt.registerOutParameter(RES_CODE_POS, Types.NUMERIC);            //rescode OUT number
	        cstmt.execute();
	        if(cstmt.getInt(RES_CODE_POS)<0)
	        	throw new SQLException("Invalid context!");
	        
			isLoaded = true;
			readFields();
		}finally{
			releaseContext();
		}
	}

	protected void releaseContext() {
		try {
			if (cstmt != null)
                cstmt.close();
		} catch (SQLException e) {
			
		} finally {
			if (conn != null && releaseConn)
				Connector.releaseConnection(conn);
		}
	}	

    public String toString() {
        StringBuffer buf = new StringBuffer();

        buf.append("\nTransaction Context");
        buf.append("\n\tAckHelpDeskNo: [");
        buf.append(getAcqHelpdeskNo());
        buf.append("]\n\tAcqID: [");
        buf.append(getAcqID());
        buf.append("]\n\tAcqManufacturer: [");
        buf.append(getAcqManufacturerID());
        buf.append("]\n\tAckSubmiterID: [");
        buf.append(getAcqSubmiterID());
        buf.append("]\n\tAcqX25No: [");
        buf.append(getAcqX25No());
        buf.append("]\n\tCardType: [");
        buf.append(getCardType());
        buf.append("]\n\tTrmAcqID: [");
        buf.append(getTrmAcqID());
        buf.append("]\n\tAcqCommMethod: [");
        buf.append(getAcqCommMethod());
        buf.append("]\n\tAcqIPHost: [");
        buf.append(getAcqIPHost());
        buf.append("]\n\tAcqIPPort: [");
        buf.append(getAcqIPPort());
        buf.append("]\n\tTrmCity: [");
        buf.append(getTrmCity());
        buf.append("]\n\tTrmCompany: [");
        buf.append(getTrmCompany());
        buf.append("]\n\tTrmID: [");
        buf.append(getTrmID());
        buf.append("]\n\tTrmManfacturer: [");
        buf.append(getTrmManufacturer());
        buf.append("]\n\tTrmModel: [");
        buf.append(getTrmModel());
        buf.append("]\n\tTrmOptions: [");
        buf.append(getTrmOptions());
        buf.append("]\n\tVUNumber: [");
        buf.append(getVUNumber());
        buf.append("]\nEnd of Context\n");

        return buf.toString();
    }
	
    public static void main(String args[]) {
        BasicConfigurator.configure();
    	Connector.setJDBCParams(
				"oracle.jdbc.driver.OracleDriver",
    			"jdbc:oracle:thin:@213.62.66.14:1521:oradb",
    			"cdsman", 
    			"manager");
		TransactionContext ctx;
        try {
            ctx = new TransactionContext("89441290000000", "00550056"); //"5404431999001785=03031010000000000429", "00550056");
			if (!ctx.isLoaded()) {
				logger.info(":-(");
			} else {
				logger.info(":-)");
				logger.info(ctx);
			}
			ctx.releaseContext();
        } catch (SQLException e) {
			logger.error(e);
        }
		return;
    }
}

