package com.commtrain.PMS.iso;

import org.jpos.iso.ISOException;
import org.jpos.iso.ISOField;
import org.jpos.iso.ISOMsg;
import org.jpos.iso.ISOPackager;

import com.commtrain.PMS.transaction.Transaction;


public class AdditionalData extends ISOMsg {

    /**
     * Default constructor.
     */
    public AdditionalData() {
        super(Transaction.ADDITIONAL_DATA);
    }

    /**
     * Return the object value associated with the given table ID
     * @param tableID the Field table ID
     * @return the field Object
     * @throws ISOException if tableID is not correct
     */
    public Object getValue(String tableID) throws ISOException {
        int fldno = AdditionalDataDefinition.getFieldNumber(tableID);
        return getComponent(fldno).getValue();
    }

    /**
     * Return the String value associated with the given ISOField table ID
     * @param tableID the Field table ID
     * @return field's String value
     */
    public String getString(String tableID) {
        int fldno = AdditionalDataDefinition.getFieldNumber(tableID);
        String s;
        try {
            Object obj = getValue(fldno);
            if (obj instanceof String) {
                s = (String) obj;
            } else {
                s = null;
            }
        } catch (ISOException e) {
            // ignore ISOException - return null
            s = null;
        }
        return s;
    }

    /**
     * Creates an ISOField associated with tableID within this ISOMsg
     * @param tableID the Field table ID
     * @param value field value
     * @throws ISOException if tableID is not correct
     * 
     */
    public void set(String tableID, String value) throws ISOException {
        int fldno = AdditionalDataDefinition.getFieldNumber(tableID);
        set(new ISOField(fldno, value));
    }

    /**
     * Replaces an AdditiomalData composite field with ISOFiled wich contains
     * packeted additional data
     * @param message the message which contains AdditionalData
     * @param packager the packager which is used to pack the field
     * @throws ISOException if some exception occurs
     * 
     */
    public static void prepack(ISOMsg message, ISOPackager packager) throws ISOException {
        if (message.hasField(Transaction.ADDITIONAL_DATA)) {
            Object additionalData = message.getValue(Transaction.ADDITIONAL_DATA);
            if (additionalData instanceof AdditionalData) {
                byte[] rawData = packager.pack((AdditionalData) additionalData);
                message.set(Transaction.ADDITIONAL_DATA, rawData);
            }
        }
    }

    /**
     * Replaces an ISOFiled wich contains packeted additional data
     * with AdditiomalData composite field
     * @param message the message which contains packeted additional data
     * @param packager the packager which is used to unpack the field
     * @throws ISOException if some exception occurs
     * 
     */
    public static void postunpack(ISOMsg message, ISOPackager packager) throws ISOException {
        if (message.hasField(Transaction.ADDITIONAL_DATA)) {
            byte[] rawData = message.getComponent(Transaction.ADDITIONAL_DATA).getBytes();
            AdditionalData additionalData = new AdditionalData();
            packager.unpack(additionalData, rawData);
            message.set(additionalData);
        }
    }
}
