package com.commtrain.PMS.iso;

import java.util.Hashtable;

import org.jpos.iso.ISOFieldPackager;
import org.jpos.iso.ISOMsg;
import org.jpos.iso.ISOPackager;

import com.commtrain.PMS.iso.packager.CVV2DataPackager;
import com.commtrain.PMS.iso.packager.CashierDataPackager;
import com.commtrain.PMS.iso.packager.PINResponseDataPackager;
import com.commtrain.PMS.isoBean.AdditionalWebShopData;
import com.commtrain.PMS.isoBean.AdditionalWebShopDataPackager;

public class AdditionalDataDefinition {
    /**
     * defititions by field number
     */
    private static Hashtable definitionsByFieldNumber;

    /**
     * defititions by table id
     */
    private static Hashtable definitionsByTableID;
    
    static {
        definitionsByFieldNumber = new Hashtable();
        definitionsByTableID = new Hashtable();
        // setup definitions;
        AdditionalDataDefinition.addDefinition(new AdditionalDataItemDefinition(
            CVV2Data.FIELD_NUMBER,
            CVV2Data.TABLE_ID,
            "CVV2 Data",
            new CVV2Data(),
            new CVV2DataPackager(),
            null));

        AdditionalDataDefinition.addDefinition(new AdditionalDataItemDefinition(
            CashierData.FIELD_NUMBER,
            CashierData.TABLE_ID,
            "Cashier Data",
            new CashierData(),
            new CashierDataPackager(),
            null));

        AdditionalDataDefinition.addDefinition(new AdditionalDataItemDefinition(
            PINResponseData.FIELD_NUMBER,
            PINResponseData.TABLE_ID,
            "PIN Response Data",
            new PINResponseData(),
            new PINResponseDataPackager(),
            null));

        AdditionalDataDefinition.addDefinition(new AdditionalDataItemDefinition(
            AdditionalWebShopData.FIELD_NUMBER,
            AdditionalWebShopData.TABLE_ID,
            "Additional Webshop Data",
            new AdditionalWebShopData(),
            new AdditionalWebShopDataPackager(),
            null));

    }

    /**
     * Adds new additional data item definition
     * @param additionalDataItem additional data item definition
     */
    public static void addDefinition(AdditionalDataItemDefinition additionalDataItem) {
        definitionsByFieldNumber.put(new Integer(additionalDataItem.getFldno()), additionalDataItem);
        definitionsByTableID.put(additionalDataItem.getTableID(), additionalDataItem);
    }

    /**
     * Gets field number by table id
     * @param tableID table id
     * @return field number
     */
    public static int getFieldNumber(String tableID) {
        return ((AdditionalDataItemDefinition) definitionsByTableID.get(tableID)).getFldno();
    }

    /**
     * Gets table id by field number
     * @param fldno field number
     * @return table id
     */
    public static String getTableID(Integer fldno) {
        return ((AdditionalDataItemDefinition) definitionsByFieldNumber.get(fldno)).getTableID();
    }

    /**
     * Gets description by field number
     * @param fldno field number
     * @return description
     */
    public static String getDescription(Integer fldno) {
        return ((AdditionalDataItemDefinition) definitionsByFieldNumber.get(fldno)).getDescription();
    }

    /**
     * Gets ISOMsg by table id
     * @param tableID table id
     * @return ISOMsg
     */
    public static ISOMsg getISOMsg(String tableID) {
        return ((AdditionalDataItemDefinition) definitionsByTableID.get(tableID)).getIsoMsg();
    }

    /**
     * Gets ISOPackager by field number
     * @param fldno field number
     * @return ISOPackager
     */
    public static ISOPackager getISOPackager(Integer fldno) {
        return ((AdditionalDataItemDefinition) definitionsByFieldNumber.get(fldno)).getIsoPackager();
    }

    /**
     * Gets ISOPackager by table id
     * @param tableID table id
     * @return ISOPackager
     */
    public static ISOPackager getISOPackager(String tableID) {
        return ((AdditionalDataItemDefinition) definitionsByTableID.get(tableID)).getIsoPackager();
    }

    /**
     * Gets ISOFieldPackager by field number
     * @param fldno field number
     * @return ISOFieldPackager
     */
    public static ISOFieldPackager getFieldPackager(Integer fldno) {
        return ((AdditionalDataItemDefinition) definitionsByFieldNumber.get(fldno)).getFieldPackager();
    }

    /**
     * Gets ISOFieldPackager by table id
     * @param tableID table id
     * @return ISOFieldPackager
     */
    public static ISOFieldPackager getFieldPackager(String tableID) {
        return ((AdditionalDataItemDefinition) definitionsByTableID.get(tableID)).getFieldPackager();
    }

}
