package com.commtrain.PMS.iso;

import java.io.IOException;
import java.io.InputStream;
import java.net.Socket;
import java.net.SocketException;
import java.util.Enumeration;
import java.util.PropertyResourceBundle;

import org.apache.log4j.BasicConfigurator;
import org.jpos.iso.ISOException;
import org.jpos.iso.ISOMsg;
import org.jpos.iso.ISOField;
import org.jpos.iso.ISOUtil;

/**
 * 
 *
 * To change this generated comment edit the template variable "typecomment":
 * Window>Preferences>Java>Templates.
 * To enable and disable the creation of type comments go to
 * Window>Preferences>Java>Code Generation.
 */
public class CARRequest extends ISOMsg {

    private static org.apache.log4j.Logger logger = org.apache.log4j.Logger.getLogger(CARRequest.class);
    
    public static final int CAR_RECORD_TYPE_SALE_MAG = 10;
    public static final int CAR_RECORD_TYPE_SALE_KBD = 11;
    public static final int CAR_RECORD_TYPE_MOTO_MAG = 40;
    public static final int CAR_RECORD_TYPE_MOTO_KBD = 41;
    public static final int CAR_RECORD_TYPE_REVERSAL = 99;
    public static final int CAR_RECORD_TYPE_VOID = 99;



    public static final int CAR_VERSION = 11;
    public static final int MANIFACTURE_ID = 12;
    public static final int TERMINAL_ID = 13;
    public static final int SEQUENCE_NO_ID = 14;
    public static final int TERMINAL_TYPE = 15;
    public static final int RECORD_TYPE = 16;
    public static final int VU_NUMBER = 17;
    public static final int CARD_DETAILS = 19;
    public static final int AMOUNT = 21;
    public static final int CURRENCY = 22;

    // int[] = {Field Name, Field Separators Count}
    protected static final int[][] fields = { { CAR_VERSION, 0 }, {
            MANIFACTURE_ID, 0 }, {
            TERMINAL_ID, 0 }, {
            SEQUENCE_NO_ID, 0 }, {
            TERMINAL_TYPE, 0 }, {
            RECORD_TYPE, 0 }, {
            VU_NUMBER, 1 }, {
            CARD_DETAILS, 1 }, {
            AMOUNT, 0 }, {
            CURRENCY, 1 }
    };
    protected static final int CAR_REQUEST_LENGTH = 90;

    public static final char FieldSeparator = 0x1C;
	
    protected String defaultCARVersion = "2";

    public CARRequest() {
        super();
        setField(CAR_VERSION, defaultCARVersion);
    }

    public CARRequest(String carVersion) {
        super();
        setField(CAR_VERSION, carVersion);
    }

    public void setField(int fldno, String value) {
        try {
            String newvalue;
            switch (fldno) {
                case CARD_DETAILS :
                    newvalue = ISOUtil.strpad(value, 40);
                    break;

                case VU_NUMBER :
                    newvalue = ISOUtil.padleft(value, 13, '0');
                    break;

                default :
                    newvalue = value;
                    break;
            }
            this.set(new ISOField(fldno, newvalue));
        } catch (ISOException e) {
            //Must not happen
        }
    }

    /**
     * @see org.jpos.iso.ISOComponent#pack()
     */
    public byte[] pack() throws ISOException {
        StringBuffer result = new StringBuffer();
        StringBuffer errors = new StringBuffer();
        int fldno;

        for (int i = 0; i < fields.length; i++) {
            fldno = fields[i][0];
            if (this.hasField(fldno)) {
                result.append(this.getString(fldno));
                int separators = fields[i][1];
                for (int j = 0; j < separators; j++) {
                    result.append(FieldSeparator);
                }
            } else {
                errors.append(", " + fldno);
            }
        }

        if (errors.length() == 0 && result.length() == CAR_REQUEST_LENGTH) {
        	logger.info(result);
            return result.toString().getBytes();
        } else if (errors.length() != 0) {
            throw new ISOException("Missing fields:" + errors.toString().substring(1) + ".");
        } else {
            throw new ISOException("Invalid CAR request length: " + result.length() + ".");
        }
    }

    public CARResponse simulateCARResponse() {
        CARResponse response = new CARResponse();

        response.setField(CARResponse.CAR_VERSION, this.getString(CARRequest.CAR_VERSION));
        response.setField(CARResponse.MANIFACTURE_ID, this.getString(CARRequest.MANIFACTURE_ID));
        response.setField(CARResponse.TERMINAL_ID, this.getString(CARRequest.TERMINAL_ID));
        response.setField(CARResponse.SEQUENCE_NO_ID, this.getString(CARRequest.SEQUENCE_NO_ID));
        String RecordType = "12";
        response.setField(CARResponse.RECORD_TYPE, RecordType);
        String ResponseIndicator = "00";
        response.setField(CARResponse.RESPONSE_INDICATOR, ResponseIndicator);
        String ResponseCode = "00";
        response.setField(CARResponse.RESPONSE_CODE, ResponseCode);
        String ApprovalCode = "142857";
        response.setField(CARResponse.APPROVAL_CODE, ApprovalCode);
        int floorLimit = 999;
        response.setFloorLimit(new Integer(floorLimit));
        if (CARResponse.RESPONSE_INDICATOR_REFERAL.equals(response.getField(CARResponse.RESPONSE_INDICATOR))) {
            String AdditionalData = "02" + (char) 0x1F + "9633053      ";
            response.setField(CARResponse.ADDITIONAL_DATA, AdditionalData);
        }

        return response;
    }
    /**
     * Sends request to the server. It will wait until response is recieved or timeout is reached.
     * Only for tests
     */
    public void send(String serverHost, int serverPort, int timeout, boolean hasTPDUHeader) {
        CARResponse responseMsg = new CARResponse();
        Socket comSocket = null;
        try {
            comSocket = new Socket(serverHost, serverPort);
            //send
            byte[] packetedMessage = this.pack();

            byte[] theMessage;
			if(hasTPDUHeader){
	            int L = header.getLength() + packetedMessage.length /* 0xA500*/;
	            byte[] theHeader = header.pack();
	            theMessage = new byte[L + 2];
	            theMessage[0] = (byte) (L >> 8);
	            theMessage[1] = (byte) (L & 0xFF);
	            System.arraycopy(theHeader, 0, theMessage, 2, theHeader.length);
	            System.arraycopy(packetedMessage, 0, theMessage, 2 + theHeader.length, packetedMessage.length);
			}else{
	            theMessage=packetedMessage;
			}

            logger.info(ISOUtil.hexString(theMessage));
            comSocket.getOutputStream().write(theMessage);
            logger.info("The message is sent.\n");
 
            long tStart = System.currentTimeMillis();
            long tFinish = tStart + timeout;
            long tCurrent = tStart + timeout;
            int available;
            //receive response length
            InputStream in = comSocket.getInputStream();
            while ((available = in.available()) < 2) {
                Thread.sleep(20);
                tCurrent = System.currentTimeMillis();
                if (tCurrent >= tFinish) {
                    throw new SocketException("Time elapsed " + (tCurrent - tStart) + " miliseconds, bytes received " + available);
                }
            }
            byte[] r = new byte[2];
            in.read(r);
            byte[] h = new byte[header.getLength()];
            System.arraycopy(r, 0, h, 0, 2);
            int l = ((r[0] & 0xFF) << 8) + r[1] & 0xFF;
            //recieve ISOResponse
            r = new byte[l];
            while ((available = in.available()) < l) {
                Thread.sleep(20);
                tCurrent = System.currentTimeMillis();
                if (tCurrent >= tFinish) {
                    throw new SocketException("Time elapsed " + (tCurrent - tStart) + " miliseconds, bytes received " + available + 2);
                }
            }
            in.read(r);
            tCurrent = System.currentTimeMillis();

            //remove header
            System.arraycopy(r, 0, h, 0, header.getLength());
            header.unpack(h);
            packetedMessage = new byte[l - header.getLength()];
            System.arraycopy(r, header.getLength(), packetedMessage, 0, packetedMessage.length);
            //            responseMsg.setPackager(packager);
            responseMsg.unpack(packetedMessage);
            String theResponse = new String(packetedMessage);
            logger.info("The response is received in " + (tCurrent-tStart) + " miliseconds.\n\n" + theResponse);
            logger.info("CAR Version: " + theResponse.substring(0, 1) + "\nManifacture-ID: " + theResponse.substring(1, 3) + "\nTerminal-ID: " + theResponse.substring(3, 9) + "\nSequence no.: " + theResponse.substring(9, 13) + "\nRecord Type: " + theResponse.substring(13, 15) + "\nResponse Indicator: " + theResponse.substring(15, 17) + "\nResponse Code: " + theResponse.substring(20, 22) + "\nApproval Code: " + theResponse.substring(30, 36) + "\nFloor Limit: " + theResponse.substring(39, 42));
            if (theResponse.length() == 60) {
                logger.info("Additional Data: " + theResponse.substring(43, 59));
            }
        } catch (Exception e) {
       		logger.error("Stack Trace", e);
        } finally {
            try {
                if (comSocket != null)
                    comSocket.close();
            } catch (IOException e) {
                logger.error("send: " + e.toString());
            }
        }
    }
    /*
     * Only for tests
     */
    public static void main(String args[]) {

        BasicConfigurator.configure();
        if (args.length != 1) {
            logger.info("Usage:\n\tCARRequest <propertiesFileName>");
            return;
        }

        PropertyResourceBundle resourceBundle = null;
        try {
            ClassLoader loader = CARRequest.class.getClassLoader();
            InputStream inp = loader.getResourceAsStream(args[0]);
            resourceBundle = new PropertyResourceBundle(inp);
        } catch (Exception e) {
            logger.error("Resource file:" + args[0] + " not found!", e);
        }

        String serverHost;
        try {
            serverHost = resourceBundle.getString("host");
        } catch (Exception e) {
            serverHost = "172.31.251.4";
        }
        logger.info("serverHost: " + serverHost);

        int serverPort = 1000;
        try {
            serverPort = Integer.parseInt(resourceBundle.getString("port"));
        } catch (Exception e) {
            serverPort = 1000;
        }
        logger.info("serverPort: " + serverPort);

        int timeout = 18000;
        try {
            timeout = Integer.parseInt(resourceBundle.getString("timeout"));
        } catch (Exception e) {
            timeout = 18000;
        }
        logger.info("timeout: " + timeout + " miliseconds");

        boolean hasTPDUHeader = true;
        try {
            hasTPDUHeader = Boolean.valueOf(resourceBundle.getString("hasTPDUHeader")).booleanValue();
        } catch (Exception e) {
        }
        logger.info("hasTPDUHeader: " + hasTPDUHeader);

        String element;
        String request;

        for (Enumeration iter = resourceBundle.getKeys(); iter.hasMoreElements();) {
           element = (String) iter.nextElement();
            if (!element.startsWith("RQ_")) {
                continue;
            }
            request = resourceBundle.getString(element);
            logger.info("\n" + element + '=' + request + "\n");
            if(!hasTPDUHeader)
            	request="00000000000000"+request;//add dummy tpduHeader
            CARRequest requestMsg = new CARRequest();
            try {
                requestMsg.unpack(request);
            } catch (ISOException e) {
            }
            if(!hasTPDUHeader)
            	requestMsg.setHeader(new byte[0]);
            requestMsg.send(serverHost, serverPort, timeout, hasTPDUHeader);
        }
    }

    /*
     * Only for tests
     */
    public int unpack(String hexMessage) throws ISOException {
        String packetedMessage = new String(ISOUtil.hex2byte(hexMessage));
        IENHeader header = new IENHeader();
        header.setId((byte) 96);
        header.setDestination(hexMessage.substring(6, 10));
        header.setSource(hexMessage.substring(10, 14));
        this.setHeader(header);
        logger.info(packetedMessage.substring(7));
        setField(CARRequest.CAR_VERSION, packetedMessage.substring(7, 8));
        setField(CARRequest.MANIFACTURE_ID, packetedMessage.substring(8, 10));
        setField(CARRequest.TERMINAL_ID, packetedMessage.substring(10, 16));
        setField(CARRequest.SEQUENCE_NO_ID, packetedMessage.substring(16, 20));
        setField(CARRequest.TERMINAL_TYPE, packetedMessage.substring(20, 24));
        setField(CARRequest.RECORD_TYPE, packetedMessage.substring(24, 26));
        setField(CARRequest.VU_NUMBER, packetedMessage.substring(26, 39));
        setField(CARRequest.CARD_DETAILS, packetedMessage.substring(40, 80));
        setField(CARRequest.AMOUNT, packetedMessage.substring(81, 93));
        setField(CARRequest.CURRENCY, packetedMessage.substring(93, 96));
        logger.info("CAR Version: " + packetedMessage.substring(7, 8)
                         + "\nManifacture-ID: " + packetedMessage.substring(8, 10)
                         + "\nTerminal-ID: " + packetedMessage.substring(10, 16)
                         + "\nSequence no.: " + packetedMessage.substring(16, 20)
                         + "\nTerminal Type: " + packetedMessage.substring(20, 24)
                         + "\nRecord Type: " + packetedMessage.substring(24, 26)
                         + "\nVU Number: " + packetedMessage.substring(26, 39)
                         + "\nCard Details: " + packetedMessage.substring(40, 80)
                         + "\nAmount: " + packetedMessage.substring(81, 93)
                         + "\nCurrency: " + packetedMessage.substring(93, 96));
        return 90;
    }

}
