package com.commtrain.PMS.iso;

import org.jpos.iso.ISOException;
import org.jpos.iso.ISOMsg;
import org.jpos.iso.ISOField;

/**
 * 
 *
 * To change this generated comment edit the template variable "typecomment":
 * Window>Preferences>Java>Templates.
 * To enable and disable the creation of type comments go to
 * Window>Preferences>Java>Code Generation.
 * 
 */
public class CARResponse extends ISOMsg {
    public static final int CAR_VERSION = 11;
    public static final int MANIFACTURE_ID = 12;
    public static final int TERMINAL_ID = 13;
    public static final int SEQUENCE_NO_ID = 14;
    public static final int RECORD_TYPE = 15;
    public static final int RESPONSE_INDICATOR = 16;
    public static final int RESPONSE_CODE = 17;
    public static final int BLANKS = 18;
    public static final int APPROVAL_CODE = 19;
    private static final int FLOOR_LIMIT = 20;
    public static final int ADDITIONAL_DATA = 21;

    public static final String RESPONSE_CODE_TRANSACTION_ACCEPTED = "01";
    public static final String RESPONSE_CODE_FORMAT_ERROR = "11";
    public static final String RESPONSE_CODE_INAVALID_TRANSACTION = "12";
    public static final String RESPONSE_CODE_CALL_REFERRAL1 = "59";
    public static final String RESPONSE_CODE_CALL_REFERRAL2 = "60";
	public static final String RESPONSE_CODE_DECLINED_CVV2 = "79";

    public static final String RESPONSE_INDICATOR_APPROVAL = "00";
    public static final String RESPONSE_INDICATOR_REFERAL = "10";
    public static final String RESPONSE_INDICATOR_DECLINE = "20";
    public static final String RESPONSE_INDICATOR_INCREASE_TO = "81";
    public static final String APPROVAL_CODE_DEFAULT = "000000";
    public static final String FLOOR_LIMIT_DEFAULT = "999";

    protected static final int CAR_RESPONSE_LENGTH1 = 43;
    protected static final int CAR_RESPONSE_LENGTH2 = 60;
    protected static final int BLANKS_OFFSET = 22;
    protected static final int BLANKS_END = BLANKS_OFFSET + 8;

    // int[] = {Field Name, Field Separators Count, Offset, Length}
    protected static final int[][] fields = { { CAR_VERSION, 0, 0, 1 }, {
            MANIFACTURE_ID, 0, 1, 2 }, {
            TERMINAL_ID, 0, 3, 6 }, {
            SEQUENCE_NO_ID, 0, 9, 4 }, {
            RECORD_TYPE, 0, 13, 2 }, {
            RESPONSE_INDICATOR, 3, 15, 2 }, {
            RESPONSE_CODE, 0, 20, 2 }, {
            APPROVAL_CODE, 3, BLANKS_END, 6 }, {
            FLOOR_LIMIT, 1, 39, 3 }, {
            ADDITIONAL_DATA, 1, CAR_RESPONSE_LENGTH1, 16 }
    };

    protected static final char SPACE = 0x20;

    public CARResponse() {
    	super();
    }

    public CARResponse(ISOMsg m) {
    	super();
        this.merge(m);
    }

    public String getField(int fldno) {
        return this.getString(fldno);
    }

    public boolean hasAdditionalData() {
        return this.hasField(ADDITIONAL_DATA);
    }

    protected void setField(int fldno, String value) {
        try {
            this.set(new ISOField(fldno, value));
        } catch (ISOException e) {
            //Must not happen
        }
    }

    /**
     * @see org.jpos.iso.ISOComponent#unpack(byte[])
     */
    public int unpack(byte[] b) throws ISOException {
        int count;

        switch (b.length) {
            case CAR_RESPONSE_LENGTH1 :
                count = fields.length - 1;
                break;
            case CAR_RESPONSE_LENGTH2 :
                count = fields.length;
                break;
            default :
                throw new ISOException("Invalid CAR response length: " + b.length + ".");
        }

        for (int i = 0; i < count; i++) {
            int offset = fields[i][2];
            int length = fields[i][3];
            int separators = fields[i][1];
            byte[] field = new byte[length];
            System.arraycopy(b, offset, field, 0, field.length);
            this.set(new ISOField(fields[i][0], new String(field)));
            offset += length;
            for (int j = offset; j < offset + separators; j++) {
                if (b[j] != CARRequest.FieldSeparator) {
                    throw new ISOException("Invalid field separator. Offset: " + j + ".");
                }
            }
        }

        for (int j = BLANKS_OFFSET; j < BLANKS_END; j++) {
            if (b[j] != SPACE) {
                throw new ISOException("Invalid reserved character. Offset: " + j + ".");
            }
        }

        String responceIndicator = this.getString(RESPONSE_INDICATOR);
//        if (this.hasAdditionalData() && !RESPONSE_INDICATOR_REFERAL.equals(responceIndicator)) {
//            throw new ISOException("Invalid combination - Response Indicator, Additional Data. Response Indicator: " + responceIndicator);
//        }
        if (RESPONSE_INDICATOR_REFERAL.equals(responceIndicator) && !this.hasAdditionalData()) {
            throw new ISOException("Invalid combination - Response Indicator, Additional Data. Response Indicator: " + responceIndicator);
        }
        return b.length;
    }
    
    public Integer getFloorLimit() {
    	try{
	    	return Integer.valueOf(getString(CARResponse.FLOOR_LIMIT));
    	}catch(Exception e){
	        return new Integer(FLOOR_LIMIT_DEFAULT);
    	}
    }

    public void setFloorLimit(Integer newFloorLimit) {
    	setField(FLOOR_LIMIT, newFloorLimit!=null?newFloorLimit.toString():null);
    }
    
}
