package com.commtrain.PMS.iso;

import java.math.BigDecimal;

import org.jpos.iso.ISOException;
import org.jpos.iso.ISOMsg;
import org.jpos.iso.ISOPackager;

import com.commtrain.PMS.transaction.Transaction;

/**
 * 
 *
 */
public class ReconciliationRequestTotals extends ISOMsg {
    
    /**
     * the field number of sales count (0)
     */
    private static final int CAPTURED_SALES_COUNT = 0;
    /**
     * the field number of sales amount (1)
     */
    private static final int SALES_AMOUNT = 1;
    /**
     * the field number of refund count (2)
     */
    private static final int REFUND_COUNT = 2;
    /**
     * the field number of refund amount (3)
     */
    private static final int REFUND_AMOUNT = 3;

    /**
     * Default constructor.
     */
    public ReconciliationRequestTotals() {
        super(Transaction.RECONCILIATION_REQUEST_TOTALS);
    }

    /**
     * Returns the creditRecordsNumber.
     * @return int
     */
    public int getCreditRecordsNumber() {
        int creditRecordsNumber = Integer.parseInt(this.getString(CAPTURED_SALES_COUNT));
        return creditRecordsNumber;
    }

    /**
     * Returns the cerditAmount.
     * @return BigDecimal
     */
    public BigDecimal getCreditAmount() {
        BigDecimal creditAmount = new BigDecimal(this.getString(SALES_AMOUNT));
        return creditAmount;
    }

    /**
     * Returns the debitRecordsNumber.
     * @return int
     */
    public int getDebitRecordsNumber() {
        int debitRecordsNumber = Integer.parseInt(this.getString(REFUND_COUNT));
        return debitRecordsNumber;
    }

    /**
     * Returns the debitAmount.
     * @return BigDecimal
     */
    public BigDecimal getDebitAmount() {
        BigDecimal debitAmount = new BigDecimal(this.getString(REFUND_AMOUNT));
        return debitAmount;
    }

    /**
     * Replaces an ReconciliationRequestTotals composite field
     * with ISOFiled wich contains packeted reconciliation request totals
     * @param message the message which contains ReconciliationRequestTotals
     * @param packager the packager which is used to pack the field
     * @throws ISOException if some exception occurs
     * 
     */
    public static void prepack(ISOMsg message, ISOPackager packager) throws ISOException {
        if (message.hasField(Transaction.RECONCILIATION_REQUEST_TOTALS)) {
            Object reconciliationRequestTotals = message.getValue(Transaction.RECONCILIATION_REQUEST_TOTALS);
            if (reconciliationRequestTotals instanceof ReconciliationRequestTotals) {
                byte[] rawData = packager.pack((ReconciliationRequestTotals) reconciliationRequestTotals);
                message.set(Transaction.RECONCILIATION_REQUEST_TOTALS, rawData);
            }
        }
    }

    /**
     * Replaces an ISOFiled wich contains packeted reconciliation request totals
     * with ReconciliationRequestTotals composite field
     * @param message the message which contains reconciliation request totals
     * @param packager the packager which is used to unpack the field
     * @throws ISOException if some exception occurs
     * 
     */
    public static void postunpack(ISOMsg message, ISOPackager packager) throws ISOException {
        if (message.hasField(Transaction.RECONCILIATION_REQUEST_TOTALS)) {
            byte[] rawData = message.getComponent(Transaction.RECONCILIATION_REQUEST_TOTALS).getBytes();
            ReconciliationRequestTotals reconciliationRequestTotals = new ReconciliationRequestTotals();
            packager.unpack(reconciliationRequestTotals, rawData);
            message.set(reconciliationRequestTotals);
        }
    }
}
