package com.commtrain.PMS.iso.filter;

import org.jpos.iso.ISOException;
import org.jpos.iso.ISOMsg;
import org.jpos.iso.ISOPackager;

import com.commtrain.PMS.iso.AdditionalData;
import com.commtrain.PMS.iso.ReconciliationRequestTotals;
import com.commtrain.PMS.iso.packager.AdditionalDataPackager;
import com.commtrain.PMS.iso.packager.ReconciliationRequestTotalsPackager;
import com.commtrain.PMS.logging.LogEntry;
import com.commtrain.PMS.transaction.Transaction;
import com.commtrain.PMS.transaction.TransactionUtils;

/**
 * 
 */
public class HypercomFilter extends PackagingFilter {

    private static org.apache.log4j.Logger logger = org.apache.log4j.Logger.getLogger(HypercomFilter.class);

    /**
     * Contains the packager for additional data field.
     */
    private ISOPackager reconciliationRequestTotalsPackager;
    /**
     * Contains the packager for reconciliation request totals field.
     */
    private ISOPackager additionalDataPackager;

    /**
     * Default constructor
     */
    public HypercomFilter() {
        super();
        this.reconciliationRequestTotalsPackager = new ReconciliationRequestTotalsPackager();
        this.additionalDataPackager = new AdditionalDataPackager();
    }

    /**
     * Packs composite fields (Additional Data)
     * @param m message which contains composite fields
     * @return message which contains only simple fields
     * @throws ISOException is there is some exception
     * @see org.jpos.iso.PackagingFilter#prepack(ISOMsg m)
     */
    protected ISOMsg prepack(ISOMsg m) throws ISOException {
        return m;
    }

    /**
     * Unpacks composite fields (Additional Data and Reconciliation request totals)
     * @param m message which contains simple fields
     * @return message which contains composite fields
     * @throws ISOException is there is some exception
     * @see org.jpos.iso.PackagingFilter#postunpack(ISOMsg m)
     * @see com.commtrain.PMS.iso.AdditionalData
     * @see com.commtrain.PMS.iso.ReconciliationRequestTotals
     */
    protected ISOMsg postunpack(ISOMsg m) throws ISOException {
        String mti = TransactionUtils.getString(m, Transaction.MESSAGE_TYPE_INDICATOR);
        if ("0100".equals(mti) || "0200".equals(mti) || "0220".equals(mti) || "0400".equals(mti)||"0420".equals(mti)) {
            AdditionalData.postunpack(m, additionalDataPackager);
        } else if ("0500".equals(mti)) {
            ReconciliationRequestTotals.postunpack(m, reconciliationRequestTotalsPackager);
            logger.info(new LogEntry("Batch", m));
        }
        return m;
    }
}
