package com.commtrain.PMS.iso.filter;

import org.jpos.iso.ISOChannel;
import org.jpos.iso.ISOException;
import org.jpos.iso.ISOFilter;
import org.jpos.iso.ISOMsg;
import org.jpos.util.LogEvent;

/**
 * 
 */
public abstract class PackagingFilter implements ISOFilter {

    /**
     * Packs and unpacks massages with composite fields.
     * If the message is incoming then its composite field are unpacked.
     * If the message is outgoing then its composite field are packed.
     * @see org.jpos.iso.ISOFilter#filter(ISOChannel, ISOMsg, LogEvent)
     */
    public ISOMsg filter(ISOChannel channel, ISOMsg m, LogEvent evt) throws VetoException {
        ISOMsg response;
        try {
            switch (m.getDirection()) {
                case ISOMsg.INCOMING :
                    response = postunpack(m);
                    break;
                case ISOMsg.OUTGOING :
                    response = prepack(m);
                    break;
                default :
                    response = m;
            }
            return response;
        } catch (ISOException e) {
            throw new VetoException(e);
        }
    }


    /**
     * Packs composite fields
     * @param m message which contains composite fields
     * @return message which contains only simple fields
     * @throws ISOException is there is some exception
     */
    protected abstract ISOMsg prepack(ISOMsg m) throws ISOException;

    /**
     * Unpacks composite fields
     * @param m message which contains simple fields
     * @return message which contains composite fields
     * @throws ISOException is there is some exception
     */
    protected abstract ISOMsg postunpack(ISOMsg m) throws ISOException;
}
