package com.commtrain.PMS.iso.packager;

import java.io.InputStream;
import java.util.Hashtable;
import java.util.Iterator;
import java.util.LinkedList;

import org.jpos.iso.ISOComponent;
import org.jpos.iso.ISOException;
import org.jpos.iso.ISOFieldPackager;
import org.jpos.iso.ISOMsg;
import org.jpos.iso.ISOPackager;
import org.jpos.iso.ISOUtil;
import org.jpos.util.LogEvent;
import org.jpos.util.Logger;

import com.commtrain.PMS.iso.AdditionalDataDefinition;

/**
 * 
 *
 */
public class AdditionalDataPackager implements ISOPackager {

    /**
     * logger
     */
    private Logger logger = null;
    /**
     * log realm
     */
    private String realm = null;

    /**
     * @see org.jpos.iso.ISOPackager#pack(ISOComponent)
     */
    public byte[] pack(ISOComponent m) throws ISOException {
        LogEvent evt = new LogEvent(this, "pack");
        evt.addMessage(m);
        try {
            if (m.getComposite() != m) {
                throw new ISOException("Can't call packager on non Composite");
            }

            LinkedList items = new LinkedList();
            Hashtable fields = m.getChildren();
            int len = 0;
            byte[] b;
            byte[] itemHeader;
            ISOComponent field;
            int itemLength;
            byte[] l;
            String slen;
            //packing of fields
            for (Iterator iter = fields.values().iterator(); iter.hasNext();) {
                field = (ISOComponent) iter.next();
                Integer fldno = (Integer) field.getKey();
                try {
                    if (field instanceof ISOMsg) {
                        ((ISOMsg) field).setPackager(AdditionalDataDefinition.getISOPackager(fldno));
                        b = field.pack();
                    } else {
                        ISOFieldPackager fieldPackager = AdditionalDataDefinition.getFieldPackager(fldno);
                        b = fieldPackager.pack(field);
                    }
                    itemLength = b.length + 2;
                    len += itemLength + 2;
                    itemHeader = new byte[4];
                    slen = ISOUtil.zeropad(Integer.toString(itemLength), 4);
                    l = ISOUtil.str2bcd(slen, true);
                    itemHeader[0] = l[0];
                    itemHeader[1] = l[1];
                    String tableID = AdditionalDataDefinition.getTableID(fldno);
                    l = tableID.getBytes();
                    itemHeader[2] = l[0];
                    itemHeader[3] = l[1];
                    items.add(itemHeader);
                    items.add(b);
                } catch (ISOException e) {
                    evt.addMessage("error packing field " + fldno.intValue());
                    evt.addMessage(field);
                    evt.addMessage(e);
                    throw e;
                }
            }

            //merging of packeted fields
            int k = 0;
            byte[] d = new byte[len];
            for (Iterator i = items.iterator(); i.hasNext();) {
                byte[] item = (byte[]) i.next();
                System.arraycopy(item, 0, d, k, item.length);
                k += item.length;
            }
            if (logger != null) { // save a few CPU cycle if no logger available
                evt.addMessage(ISOUtil.hexString(d));
            }
            return d;
        } catch (ISOException e) {
            evt.addMessage(e);
            throw e;
        } finally {
            Logger.log(evt);
        }
    }

    /**
     * @see org.jpos.iso.ISOPackager#unpack(ISOComponent, byte[])
     */
    public int unpack(ISOComponent m, byte[] b) throws ISOException {
        LogEvent evt = new LogEvent(this, "unpack");
        try {
            if (m.getComposite() != m) {
                throw new ISOException("Can't call packager on non Composite");
            }
            if (logger != null) { // save a few CPU cycle if no logger available
                evt.addMessage(ISOUtil.hexString(b));
            }

            int consumed = 0;
            int len;
            String tableID;
            int fldno ;
            ISOMsg defaultMsg;
            ISOFieldPackager fieldPackager;
            ISOPackager defaultPackager;
            ISOComponent c = null;
            while (consumed < b.length) {
                len = Integer.parseInt(ISOUtil.bcd2str(b, consumed, 4, false)) - 2;
                consumed += 2;
                tableID = new String(b, consumed, 2);
                consumed += 2;
                fldno = AdditionalDataDefinition.getFieldNumber(tableID);
                byte[] d = new byte[len];
                System.arraycopy(b, consumed, d, 0, len);
                defaultMsg = AdditionalDataDefinition.getISOMsg(tableID);
                if (defaultMsg != null) {
                    c = (ISOMsg) defaultMsg.clone();
                    defaultMsg = AdditionalDataDefinition.getISOMsg(tableID);
                    defaultPackager = AdditionalDataDefinition.getISOPackager(tableID);
                    int msglen = defaultPackager.unpack(c, d);
                    consumed += msglen;
                }
                fieldPackager = AdditionalDataDefinition.getFieldPackager(tableID);
                if (fieldPackager != null) {
                    c = fieldPackager.createComponent(fldno);
                    consumed += fieldPackager.unpack(c, d, 0);
                }
//                if (logger != null) {
//                    String description = AdditionalDataDefinition.getDescription(new Integer(fldno));
//                    evt.addMessage("<unpack fld=\"" + fldno + "\" \"" + description + "\">");
//                    if (c.getValue() instanceof ISOMsg) {
//                        evt.addMessage(c.getValue());
//                    } else {
//                        evt.addMessage("  <value>" + c.getValue().toString() + "</value>");
//                    }
//                    evt.addMessage("</unpack>");
//                }
                m.set(c);
            }
            if (b.length != consumed) {
                evt.addMessage("WARNING: unpack len=" + b.length + " consumed=" + consumed);
            }
            evt.addMessage(m);
            return consumed;
        } catch (ISOException e) {
            evt.addMessage(e);
            throw e;
        } catch (Exception e) {
            evt.addMessage(e);
            throw new ISOException(e);
        } finally {
            Logger.log(evt);
        }
    }
    
    public void unpack (ISOComponent m, InputStream in) throws ISOException{
    	throw new ISOException("unpack (ISOComponent m, InputStream in) Not implemented! Contact IP/VA");
    	
    }
    

    /**
     * @see org.jpos.iso.ISOPackager#getFieldDescription(ISOComponent, int)
     */
    public String getFieldDescription(ISOComponent m, int fldNumber) {
        return AdditionalDataDefinition.getDescription(new Integer(fldNumber));
    }

    /**
     * @see org.jpos.util.LogSource#setLogger(Logger, String)
     */
    public void setLogger(Logger logger, String realm) {
        this.logger = logger;
        this.realm = realm;
    }

    /**
     * @see org.jpos.util.LogSource#getRealm()
     */
    public String getRealm() {
        return realm;
    }

    /**
     * @see org.jpos.util.LogSource#getLogger()
     */
    public Logger getLogger() {
        return logger;
    }
}