package com.commtrain.PMS.iso.packager;

import java.io.InputStream;

import org.jpos.iso.ISOComponent;
import org.jpos.iso.ISOException;
import org.jpos.iso.ISOField;
import org.jpos.iso.ISOPackager;
import org.jpos.util.Logger;

import com.commtrain.PMS.iso.CARRequest;

/**
 * 
 */
public class CARPackager implements ISOPackager {

    private static org.apache.log4j.Logger logger = org.apache.log4j.Logger.getLogger(CARPackager.class);
    
    public static final int CAR_VERSION = 11;
    public static final int MANIFACTURE_ID = 12;
    public static final int TERMINAL_ID = 13;
    public static final int SEQUENCE_NO_ID = 14;
    public static final int RECORD_TYPE = 15;
    public static final int RESPONSE_INDICATOR = 16;
    public static final int RESPONSE_CODE = 17;
    public static final int BLANKS = 18;
    public static final int APPROVAL_CODE = 19;
    public static final int FLOOR_LIMIT = 20;
    public static final int ADDITIONAL_DATA = 21;

    protected static final int CAR_RESPONSE_LENGTH1 = 43;
    protected static final int CAR_RESPONSE_LENGTH1A = CAR_RESPONSE_LENGTH1 + 1;
    protected static final int CAR_RESPONSE_LENGTH2 = 60;
    protected static final int BLANKS_OFFSET = 22;
    protected static final int BLANKS_END = BLANKS_OFFSET + 8;

    // int[] = {Field Name, Field Separators Count, Offset, Length}
    protected static final int[][] fields = { { CAR_VERSION, 0, 0, 1 }, {
            MANIFACTURE_ID, 0, 1, 2 }, {
            TERMINAL_ID, 0, 3, 6 }, {
            SEQUENCE_NO_ID, 0, 9, 4 }, {
            RECORD_TYPE, 0, 13, 2 }, {
            RESPONSE_INDICATOR, 3, 15, 2 }, {
            RESPONSE_CODE, 0, 20, 2 }, {
            APPROVAL_CODE, 3, BLANKS_END, 6 }, {
            FLOOR_LIMIT, 1, 39, 3 }, {
            ADDITIONAL_DATA, 1, CAR_RESPONSE_LENGTH1, 16 }
    };

    protected static final char SPACE = 0x20;

    protected static final String RESPONSE_INDICATOR_REFERAL = "10";

    /**
     * Empty implementation
     * @param   m   the Component to pack
     * @return      Message image
     * @throws ISOException if some exception occurs
     * @see org.jpos.iso.ISOPackager#pack(ISOComponent)
     */
    public byte[] pack(ISOComponent m) throws ISOException {
        return null;
    }

    /**
     * Unpacks CAR response
     * @param m the Container of this message
     * @param b CAR response image
     * @return consumed bytes
     * @throws ISOException if some exception occurs
     * @see org.jpos.iso.ISOPackager#unpack(ISOComponent, byte[])
     */
    public int unpack(ISOComponent m, byte[] b) throws ISOException {
        logger.info(new String(b));

        int count;

        switch (b.length) {
            case CAR_RESPONSE_LENGTH1 :
                count = fields.length - 1;
                break;
            case CAR_RESPONSE_LENGTH1A :
                count = fields.length - 1;
                break;
            case CAR_RESPONSE_LENGTH2 :
                count = fields.length;
                break;
            default :
                throw new ISOException("Invalid CAR response length: " + b.length + ".");
        }

        for (int i = 0; i < count; i++) {
            int offset = fields[i][2];
            int length = fields[i][3];
            int separators = fields[i][1];
            byte[] field = new byte[length];
            System.arraycopy(b, offset, field, 0, field.length);
            m.set(new ISOField(fields[i][0], new String(field)));
            offset += length;
            for (int j = offset; j < offset + separators; j++) {
                if (b[j] != CARRequest.FieldSeparator) {
                    throw new ISOException("Invalid field separator. Offset: " + j + ".");
                }
            }
        }

        for (int j = BLANKS_OFFSET; j < BLANKS_END; j++) {
            if (b[j] != SPACE) {
                throw new ISOException("Invalid reserved character. Offset: " + j + ".");
            }
        }

/*        String responceIndicator = ((ISOMsg)m).getString(RESPONSE_INDICATOR);
        if (((ISOMsg)m).hasField(ADDITIONAL_DATA) && !RESPONSE_INDICATOR_REFERAL.equals(responceIndicator)) {
            throw new ISOException("Invalid combination - Response Indicator, Additional Data. Response Indicator: " + responceIndicator);
        }*/
        return b.length;
    }

    public void unpack (ISOComponent m, InputStream in) throws ISOException{
    	throw new ISOException("unpack (ISOComponent m, InputStream in) Not implemented! Contact IP/VA");
    }

    /**
     * @see org.jpos.iso.ISOPackager#getFieldDescription(ISOComponent, int)
     */
    public String getFieldDescription(ISOComponent m, int fldNumber) {
        return null;
    }

    /**
     * @see org.jpos.util.LogSource#setLogger(Logger, String)
     */
    public void setLogger(Logger logger, String realm) {
    }

    /**
     * @see org.jpos.util.LogSource#getRealm()
     */
    public String getRealm() {
        return null;
    }

    /**
     * @see org.jpos.util.LogSource#getLogger()
     */
    public Logger getLogger() {
        return null;
    }
}
