package com.commtrain.PMS.isoBean;

import java.util.BitSet;

import org.jpos.iso.ISOBitMapPackager;
import org.jpos.iso.ISOComponent;
import org.jpos.iso.ISOException;

/**
 * ISOFieldPackager Simple Binary Bitmap
 *
 * 
 * @see ISOComponent
 * @see ISOBitMapPackager
 */
public class IFB_SIMPLE_BITMAP extends ISOBitMapPackager {

    public IFB_SIMPLE_BITMAP(int len, String description) {
        super(len, description);
    }
    /**
     * @see org.jpos.iso.ISOFieldPackager#getMaxPackedLength()
     */
    public int getMaxPackedLength() {
        return (getLength() + 7) >> 3;
    }

    /**
     * @see org.jpos.iso.ISOFieldPackager#pack(ISOComponent)
     */
    public byte[] pack(ISOComponent c) throws ISOException {

        BitSet b = (BitSet) c.getValue();
        byte[] d = new byte[getMaxPackedLength()];

        int len = Math.min(getLength(), b.length());

        for (int i = 0; i < len; i++) {
            if (b.get(i + 1)) {
                d[i >> 3] |= (0x80 >> (i % 8));
            }
        }

        return d;
    }

    /**
     * @see org.jpos.iso.ISOFieldPackager#unpack(ISOComponent, byte[], int)
     */
    public int unpack(ISOComponent c, byte[] b, int offset) throws ISOException {
        
        int len = getLength();

        BitSet bmap = new BitSet(len);
        
        for (int i = 0; i < len; i++) {
            if (((b[offset + (i >> 3)]) & (0x80 >> (i % 8))) > 0) {
                bmap.set(i + 1);
            }
        }
        
        c.setValue(bmap);
        
        return (getMaxPackedLength());
    }

}
