package com.commtrain.PMS.isoBean;

import java.util.*;

import org.apache.log4j.Category;
import org.apache.log4j.Logger;
import org.apache.log4j.Priority;
import org.apache.log4j.helpers.NullEnumeration;
import org.apache.log4j.PropertyConfigurator;
import org.jpos.iso.*;
import org.jpos.iso.channel.NACChannel;

import com.commtrain.PMS.logging.Logger4J;
import com.commtrain.PMS.Props;
import com.commtrain.PMS.application.Application;
import com.commtrain.PMS.iso.AdditionalData;
import com.commtrain.PMS.iso.CVV2Data;
import com.commtrain.PMS.iso.IENHeader;
import com.commtrain.PMS.iso.packager.AdditionalDataPackager;
import com.commtrain.PMS.iso.packager.HypercomPackager;
import com.commtrain.PMS.transaction.*;

/**
 * Lifecicle of the bean is the following: <BR>
 * <BR>
 * 1) create new ISOCom() <BR>
 * 2) setServerHost(String), setTimeout(int) and setServerPort(int). There are defaults for port:700
 *    and for timeout:15000 (15 sec) if you don't set them. <BR>
 * 3) setTransArt(), setOrderID, etc..... set all needed fields in request <BR>
 * 4) send() send request <BR>
 * 5) getAuthRespCode() to check if the request is properly processed. getErrorMessages() for detailed error description<BR>
 * 6) if processed OK -> getTime(), getDate() ... and all needed fields <BR>
 * 7) Goto 2), 3) or don't use it anymore <BR>
 * 
 */

public class ISOCom {

    private static final int MESSAGE_TYPE_INDICATOR = Transaction.MESSAGE_TYPE_INDICATOR;
    private static final int PAN_PRIMARY_ACC_NUM = Transaction.PAN_PRIMARY_ACC_NUM;
    private static final int PROCESSING_CODE = Transaction.PROCESSING_CODE;
    private static final int SYSTEM_TRACE_AUDIT_NUM = Transaction.SYSTEM_TRACE_AUDIT_NUM;
    private static final int TIME_LOCAL_TRANSACTION = Transaction.TIME_LOCAL_TRANSACTION;
    private static final int DATE_LOCAL_TRANSACTION = Transaction.DATE_LOCAL_TRANSACTION;
    private static final int DATE_EXPIRATION = Transaction.DATE_EXPIRATION;
    private static final int POS_ENTRY_MODE = Transaction.POS_ENTRY_MODE;
    private static final int NET_INTERNATIONAL_IDENTIFIEER = Transaction.NET_INTERNATIONAL_IDENTIFIEER;
    private static final int POS_CONDITION_CODE = Transaction.POS_CONDITION_CODE;
    private static final int TRACK_2_DATA = Transaction.TRACK_2_DATA;
    private static final int RETRIEVAL_REFERENCE_NUM = Transaction.RETRIEVAL_REFERENCE_NUM;
    private static final int AUTHORIZATION_IDENTIFICATION_RESPONSE = Transaction.AUTHORIZATION_IDENTIFICATION_RESPONSE;
    private static final int RESPONSE_CODE = Transaction.RESPONSE_CODE;
    private static final int TERMINAL_IDENTIFICATION = Transaction.TERMINAL_IDENTIFICATION;
    private static final int CARD_ACCEPTOR_IDENTIFICATION_CODE = Transaction.CARD_ACCEPTOR_IDENTIFICATION_CODE;
    private static final int TRACK_1_DATA = Transaction.TRACK_1_DATA;
    private static final int SECURITY_CONTROL_INFO = Transaction.SECURITY_CONTROL_INFO;
    private static final int INVOICE_ECR_REFNO = Transaction.INVOICE_ECR_REFNO;
    private static final int ADDITIONAL_DATA = Transaction.ADDITIONAL_DATA;
	private static final int ORIGINAL_AMOUNT = Transaction.ORIGINAL_AMOUNT;
	private static final int DISCOUNT = Transaction.DISCOUNT;


    /**
     * Field numbers of the required fields.
     */
    private static final int[] REQUIREQ_FIELDS = { 0, 2, 3, 4, 11, 14, 22, 24, 25, 41, 62, 63 };

    /**
     * Used in communication with different shops
     */
    private static Hashtable muxHashTable = new Hashtable(5);

    /** 
     * The message packager.
     */
    private static HypercomPackager packager = new HypercomPackager();

    private static final String MAIN_CATEGORY = "ISOCom";
    private static final String TRANSACTION_CATEGORY = MAIN_CATEGORY + ".Transactions";
    private static final String MUX_CATEGORY = MAIN_CATEGORY + ".MUX";
    private static Logger rootLogger = Logger.getLogger(MAIN_CATEGORY);
    private static Logger logger = Logger.getLogger(MAIN_CATEGORY + ".Bean");
    private static Logger transactionLogger = Logger.getLogger(TRANSACTION_CATEGORY);
    private static Logger debugLogger = Logger.getLogger(MAIN_CATEGORY + ".Debug");

    /**
     * Default server name 
     */
    private String serverHost;

    /**
     * Default server port 
     */
    private int serverPort = 700;

    /**
     * Default timeout (15 seconds)
     */
    private int timeout = 15000;

    /**
     * Associated ISO Request
     */
    private ISOMsg requestMsg = new ISOMsg();

    /**
     * Additional data component in ISO requiest
     */
    private AdditionalData additionalData = new AdditionalData();

    /**
     * Additional webshop data in additional data component
     */
    private AdditionalWebShopData additionalWebShopData = new AdditionalWebShopData();

    /**
     * CVV2 data in additional data component
     */
    private CVV2Data cvv2Data = new CVV2Data();

    /**
     * Associated ISO Response
     */
    private ISOMsg responseMsg = new ISOMsg();

    /**
     * Logger
     */
    private static org.jpos.util.Logger jPOSLogger = new Logger4J(Priority.INFO_INT);

    /**
     * Gets the serverHost
     * @return Returns a String
     */
    public String getServerHost() {
        return serverHost;
    }
    /**
     * Sets the serverHost
     * @param serverHost The serverHost to set
     */
    public void setServerHost(String serverHost) {
        this.serverHost = serverHost;
    }

    /**
     * Gets the serverPort
     * @return Returns a int
     */
    public int getServerPort() {
        return serverPort;
    }
    /**
     * Sets the serverPort
     * @param serverPort The serverPort to set
     */
    public void setServerPort(int serverPort) {
        this.serverPort = serverPort;
    }

    /**
     * Gets the timeout
     * @return Returns a long
     */
    public int getTimeout() {
        return timeout;
    }

    /**
     * Sends request to the server. It will wait until response is recieved or timeout is reached.
     * Get Response object with @see getResponse()
     */
    public void send() {
       // setNII(Integer.toString(Application.WEBSHOP_NII));

        // Add composite fields
        if (additionalWebShopData.getChildren().size() > 0) {
            try {
                additionalData.set(additionalWebShopData);
            } catch (ISOException e) {
                appendErrorMessage("send: additionalData.set(additionalWebShopData).");
            }
        }
        
        if (additionalData.getChildren().size() > 0) {
            try {
                setField(ADDITIONAL_DATA, (new AdditionalDataPackager()).pack(additionalData));
            } catch (ISOException e) {
                appendErrorMessage("send: setField(ADDITIONAL_DATA, additionalData.pack()).");
            }
        }

        checkRequestMessage();
        if (getErrors() > 0) {
            appendErrorMessage("send: Execution refused due to previous errors.");
            debugLogger.error(errorMessages);
            return;
        }

        ISORequest isoRequest = new ISORequest(requestMsg);
        transactionLogger.info("Trying to send ISORequest");
        isoRequest.setLogger(getLogger(), TRANSACTION_CATEGORY);
       
        ISOMUX isoMux = getMUX(serverHost, serverPort);
        if (isoMux != null) {
            isoMux.queue(isoRequest);
            responseMsg = isoRequest.getResponse(timeout);
        }

        if (responseMsg == null) {
            responseMsg = new ISOMsg();
            responseMsg.setPackager(packager);
            appendErrorMessage("send: " + "Response not received!");
            logger.error(errorMessages);
        }

    }

    static {
        try {
            boolean isLoggingConfigured = false;
            for (Category c = rootLogger; c != null; c = c.getParent()) {
                Enumeration appenders = c.getAllAppenders();
                if (!(appenders instanceof NullEnumeration)) {
                    System.out.println("ISOCom - logging is configured by " + c.getName());
                    isLoggingConfigured = true;
                    break;
                } else if (!c.getAdditivity()) {
                    break;
                }
            }
            if (!isLoggingConfigured) {
                Props.setFileName("ISOCom.properties");
                PropertyConfigurator.configureAndWatch(
                    Props.getFilePath(),
                    Props.getLong("log4jrefresh", 300000));
            }
            logger.info("ISOCom started");
        } catch (Exception e) {
            System.err.println("ISOCom - logging is not configured: " + e);
        }
    }

    /**
     * Gets ISOMUX object
     * @param hostName server side host name
     * @param port server side port
     * @return ISOMUX object
     */
    protected static synchronized ISOMUX getMUX(String hostName, int port) {
        String key = hostName + ':' + port;
        ISOMUX isoMux;
        isoMux = (ISOMUX) muxHashTable.get(key);
        if (isoMux == null) {
            IENHeader ienHeader = new IENHeader();
            //ienHeader.setId(IENHeader.HYPERCOM_ID);

            ServerChannel channel = new NACChannel(hostName, port, packager, ienHeader.pack());
            isoMux = new ISOMUX(channel);
            isoMux.setLogger(getLogger(), MUX_CATEGORY);
            Thread t = new Thread(isoMux);
            t.start();

            muxHashTable.put(key, isoMux);
        }
        return isoMux;
    }

    /**
     * Sets the timeout
     * @param timeout The timeout to set in milliseconds. Default is 15000 (15 sec)
     */
    public void setTimeout(int timeout) {
        this.timeout = timeout;
    }

    /**
     * Sets the Invoice Number/ECR Ref.# in the ISO Request
     * @param invoiceNo Invoice Number/ECR Ref.#
     */
    public void setOrderID(String invoiceNo) {
        setField(INVOICE_ECR_REFNO, invoiceNo);
    }

    /**
     * Sets Shop(TerminalID) in the ISO Request
     * @param shop Shop(TerminalID)
     */
    public void setShop(String shop) {
        setField(TERMINAL_IDENTIFICATION, shop);
    }

    /**
     * Sets Record Type in the ISO Request
     * @param recordType could be RESERVATION, BUCHUNG, TEILBUCHUNG, TEILBUCHUBG_STORNO, GUTSCHRIFT
     */
    public void setTransArt(int recordType) {
        setTransArt(Integer.toString(recordType));
    }

    /**
     * Sets Record Type in the ISO Request
     * @param recordType could be "10","20","30","40","50", "60",
     */
    public void setTransArt(String recordType) {
        String[] result = TransactionMapping.getClientAndISOCodes(recordType);
        String mti = result[1];
        String processingCode = result[2];
        if (mti == null && processingCode == null) {
            appendErrorMessage("setTransArt: Invalid recordType.");
            return;
        }
        setField(MESSAGE_TYPE_INDICATOR, mti);
        setField(PROCESSING_CODE, processingCode);
    }

    /**
     * Sets Primary Acct Number in the ISO Request
     * @param cardNo primary account number
     */
    public void setCardNo(String cardNo) {
        setField(PAN_PRIMARY_ACC_NUM, cardNo);
    }

    /**
     * Sets Date Expiration. "0304" for March 2004 in the ISO Request.
     * @param expDateStr should be 4 characters 
     */
    public void setExpDate(String expDateStr) {
        if (expDateStr != null && expDateStr.length() == 4) {
            expDateStr = expDateStr.substring(2) + expDateStr.substring(0, 2);
        }
        setField(DATE_EXPIRATION, expDateStr);
    }

    /**
     * Sets Additional Data CVV2 in the ISO Request.
     * @param vv2Str should be 4 characters
     */
    public void setVV2Data(String vv2Str) {
        String v;
        int l = vv2Str.length();
        if (l > 4) {
            appendErrorMessage("setVV2Data: The length of CVV2 is greater then 4.");
            return;
        } else if (l < 4) {
            try {
                v = ISOUtil.padleft(vv2Str, 4, ' ');
            } catch (ISOException e) {
                appendErrorMessage("setVV2Data: " + e.toString());
                return;
            }
        } else {
            v = vv2Str;
        }
        try {
            cvv2Data.setFlag1("1");
            cvv2Data.setFlag2("0");
            cvv2Data.setCVV2Value(v);
            additionalData.set(cvv2Data);
        } catch (ISOException e) {
            appendErrorMessage("setVV2Data: " + e.toString());
        }
    }

    /**
     * Sets amount Trans in the ISO Request.
     * @param ammount is number of raps in CH or cents in US. Send 1250 for 12.50CHF
     */
    public void setAmount(String ammount) {
        setField(4, ammount);
    }

    /**
     * Sets Systems Trace No in the ISO Request.
     * @param orderNo2 systems trace audit number
     */
    public void setOrderNo2(String orderNo2) {
        setField(SYSTEM_TRACE_AUDIT_NUM, orderNo2);
    }

    /**
     * Sets POS Entry Mode in the ISO Request.
     * @param entryMode "01" Key Entry, "02" Mag Entry
     */
    public void setEntryMode(String entryMode) {
        String v = "";
        String errorText = "";
        int l = entryMode.length();
        if (l > 2) {
            errorText = "The length of entryMode is greater then 2.";
        } else if (l < 2) {
            try {
                v = ISOUtil.padleft(entryMode, 2, '0');
            } catch (ISOException e) {
                errorText = e.toString();
            }
        } else {
            v = entryMode;
        }
        if ((errorText.length() == 0) && !v.equals("01") & !v.equals("02")) {
            errorText = "entryMode must be 01 or 02.";
        }
        if (errorText.length() == 0) {
            setField(POS_ENTRY_MODE, v + "2");
        } else {
            appendErrorMessage("setEntryMode: " + errorText);
        }
    }

    /**
     * Sets NII code in the ISO Request.
     * @param nii length max 3
     */
    public void setNII(String nii) {
        setField(NET_INTERNATIONAL_IDENTIFIEER, nii);
    }

    /**
     * Sets POS Codition Code in the ISO Request.
     * @param conditionCode length max 2
     */
    public void setConditionCode(String conditionCode) {
        setField(POS_CONDITION_CODE, conditionCode);
    }

    /**
     * Sets Track2 data in the ISO Request.
     * @param track2 Needed if POS Entry Mode is Mag Entry (02)
     */
    public void setTrack2(String track2) {
        setField(TRACK_2_DATA, track2);
    }

    /**
     * Sets RetrievalRef in the ISO Request.
     * @param retrievalRefNo retrieval reference number
     */
    public void setRetrievalRef(String retrievalRefNo) {
        setField(RETRIEVAL_REFERENCE_NUM, retrievalRefNo);
    }

    /**
     * Sets track1Data in the ISO Request.
     * @param track1Data Needed only if POS Entry Mode is Mag Entry (02)
     */
    public void setTrack1(String track1Data) {
        setField(TRACK_1_DATA, track1Data);
    }

    /**
     * Sets security control in the ISO Request.
     * @param securityControl security control as keyed in
     */
    public void setSecurityControl(String securityControl) {
        setField(SECURITY_CONTROL_INFO, securityControl);
    }

    /**
     * Sets Total Amount in the ISO Request. 
     * @param bruttoAmount total amount in the ISO Request
     */
    public void setPosAmountBrutto(String bruttoAmount) {
        setField(ORIGINAL_AMOUNT, bruttoAmount);
    }

    /**
     * Sets Discount Amount in the ISO Request.
     * @param rabattAmount discount amount
     */
    public void setPosAmountRabatt(String rabattAmount) {
        setField(DISCOUNT, rabattAmount);
    }

    /**
     * Sets Number of postions in the ISO Request. Not implented yet.
     * @param noOfPos number of postions in the ISO Request
     */
    public void setNoOfPos(String noOfPos) {
        try {
            additionalWebShopData.setNoOfPos(noOfPos);
        } catch (ISOException e) {
           appendErrorMessage("setNoOfPos: " + e.toString());
        }
        
    }

    /**
     * Sets Postion number in the ISO Request. Not implented yet.
     * @param noPos postion number in the ISO Request
     */
    public void setNoPos(String noPos) {
        try {
            additionalWebShopData.setNoPos(noPos);
        } catch (ISOException e) {
           appendErrorMessage("setNoPos: " + e.toString());
        }
    }

    /**
     * Sets Rayon in the ISO Request. Not implented yet.
     * @param rayon rayon in the ISO Request
     */
    public void setRayon(String rayon) {
        try {
            additionalWebShopData.setRayon(rayon);
        } catch (ISOException e) {
           appendErrorMessage("setRayon: " + e.toString());
        }
    }

    /**
     * Gets Shop(TerminalID) from the ISO Response.
     * @return shop(TerminalID)
     */
    public String getShop() {
        return getField(TERMINAL_IDENTIFICATION);
    }

    /**
     * Gets Record Type from the ISO Response.
     * @return record type of the ISO Request
     */
    public String getTransArt() {
        String mti = getField(MESSAGE_TYPE_INDICATOR);
        String processingCode = getField(PROCESSING_CODE);
        try {
            mti = ISOUtil.padleft(Integer.toString(Integer.parseInt(mti) - 10), 4, '0');
        } catch (ISOException e) {
            mti = "";
        }

        String transArt = TransactionMapping.getRecordType(Integer.toString(Application.WEBSHOP_ID), mti, processingCode);

        return transArt;
    }

    /**
     * Get time local trans from the ISO Response.
     * @return "184019" for 18:40:19
     */
    public String getTime() {
        return getField(TIME_LOCAL_TRANSACTION);
    }

    /**
     * Get date local trans from the ISO Response.
     * @return "0711" for 11 July 2002
     */
    public String getDate() {
        return getField(DATE_LOCAL_TRANSACTION);
    }

    /**
     * Get Systems Trace No from the ISO Response.
     * @return Systems Trace No 
     */
    public String getOrderNo2() {
        return getField(SYSTEM_TRACE_AUDIT_NUM);
    }

    /**
     * Get NII code from the ISO Response.
     * @return NII code 
     */
    public String getNII() {
        return getField(NET_INTERNATIONAL_IDENTIFIEER);
    }

    /**
     * Get RetrievalRef from the ISO Response.
     * @return RetrievalRef
     */
    public String getRetrievalRef() {
        return getField(RETRIEVAL_REFERENCE_NUM);
    }

    /**
     * Get authentication ID response from the ISO Response.
     * @return 0=OK, >0 NotOK
     */
    public String getAuthResp() {
        return getField(AUTHORIZATION_IDENTIFICATION_RESPONSE);
    }

    /**
     * Get Response Code from the ISO Response.
     * @return 00=APPROVED, 43=Stolen Card
     */
    public String getAuthRespCode() {
        if (getErrors() == 0) {
            return getField(RESPONSE_CODE);
        } else {
            return "??";
        }
    }

    /**
     * Get Response Code from the ISO Response as CAR Response Code.
     * @return 00=APPROVED, 43=Stolen Card
     */
    public String getCARAuthRespCode() {
        String carRespCode;
        int isoRespCode;
        if (getErrors() != 0) {
            return "??";
        }
        try {
            isoRespCode = Integer.parseInt(getField(RESPONSE_CODE));
        } catch (Exception e) {
            isoRespCode = 79;
        }
        switch (isoRespCode) {
            case 0:
                carRespCode = "01";
                break;
            case 2:
                carRespCode = "60";
                break;
            case 12:
                carRespCode = "12";
                break;
            case 13:
                carRespCode = "13";
                break;
            case 30:
                carRespCode = "11";
                break;
            case 54:
                carRespCode = "54";
                break;
            case 58:
                carRespCode = "12";
                break;
            case 79:
                carRespCode = "50";
                break;
            default:
                carRespCode = "50";
        }
        return carRespCode;
    }

    /**
     * Get Card Acquirer ID from the ISO Response.
     * @return CardAcqID
     */
    public String getCardAcqID() {
        return getField(CARD_ACCEPTOR_IDENTIFICATION_CODE);
    }

    /**
     * Tests basic functions.
     * @param argv not used
     */
    public static void main(String argv[]) {

        for (int i = 0; i < 10; i++) {
            ISOCom testBean = new ISOCom();
            testBean.setServerHost("localhost");
            testBean.setServerPort(800);
            testBean.setTimeout(1000000);
            testBean.setAmount("1250");
            testBean.setCardNo("4921130200000035");
            testBean.setConditionCode("18"); //or 18 for Track1 and track 2
            testBean.setEntryMode("2");
            testBean.setExpDate("0304");
            testBean.setNII("99");
            testBean.setOrderID("412938");
            testBean.setOrderNo2("12345");
            testBean.setRetrievalRef("000000000012");
            testBean.setSecurityControl("2345678");
            testBean.setShop("04550020");
            testBean.setTrack1("Track1Data");
            testBean.setTrack2("4539780352675010=04042011590000000000");
            testBean.setVV2Data("567");
            testBean.setTransArt("10");
            testBean.setPosAmountBrutto("1300");
            testBean.setPosAmountRabatt("50");
            testBean.setNoOfPos("12");
            testBean.setNoPos("34");
            testBean.setRayon("7");
            testBean.send();
            if (testBean.getErrors() > 0) {
                logger.error(testBean.errorMessages);
            }
            logger.info("AuthResp = " + testBean.getAuthResp());
            /*        logger.info("AuthRespCode = " + testBean.getAuthRespCode());
                    logger.info("CardAcqID = " + testBean.getCardAcqID());
                    logger.info("Date = " + testBean.getDate());
                    logger.info("NII = " + testBean.getNII());
                    logger.info("OrderNo2 = " + testBean.getOrderNo2());
                    logger.info("RetrievalRef = " + testBean.getRetrievalRef());
                    logger.info("Shop = " + testBean.getShop());
                    logger.info("Time = " + testBean.getTime());
                    logger.info("TransArt = " + testBean.getTransArt());
            */
        }
        //        testBean.send();
        //        logger.info("AuthResp = " + testBean.getAuthResp());

    }

    /** 
     * All error messages.
     */

    private ArrayList errorMessages = new ArrayList();

    /**
     * Append an error message.
     * @param errorMessage message to be added
     */
    private void appendErrorMessage(String errorMessage) {
        errorMessages.add(errorMessage);
    }

    /**
     * Gets the errors count
     * @return Returns a int
     */

    public int getErrors() {
        return errorMessages.size();
    }

    /**
     * Gets the error messages
     * @return Returns a String[]
     */
    public String[] getErrorMessages() {
        return (String[]) errorMessages.toArray(new String[0]);
    }

    /**
     * Checks for required input fields.
     */
    private void checkRequestMessage() {
        if (packager == null) {
            return;
        }
        int fldno;
        for (int i = 0; i < REQUIREQ_FIELDS.length; i++) {
            fldno = REQUIREQ_FIELDS[i];
            if (!requestMsg.hasField(fldno)) {
                appendFieldError(fldno);
            }
        }
        //if (requestMsg.hasField(0) && (requestMsg.getString(0) == "0220") && !requestMsg.hasField(53)) {
        //    appendFieldError(53);
        //}
        if (requestMsg.hasField(POS_ENTRY_MODE) && requestMsg.getString(POS_ENTRY_MODE).substring(0, 2).equals("02")) {
            if (!requestMsg.hasField(TRACK_2_DATA)) {
                appendFieldError(TRACK_2_DATA);
            }
            if (!requestMsg.hasField(TRACK_1_DATA)) {
                appendFieldError(TRACK_1_DATA);
            }
        }
        if (requestMsg.hasField(PROCESSING_CODE) && (requestMsg.getString(PROCESSING_CODE).equals("020000")
                || (requestMsg.getString(PROCESSING_CODE).equals("020001"))) && !requestMsg.hasField(RETRIEVAL_REFERENCE_NUM)) {
            appendFieldError(RETRIEVAL_REFERENCE_NUM);
        }
    }

    /**
     * Appends a field validation error message.
     * @param fldno field number of the validated field.
     * @param e the validation exception.
     */
    private void appendFieldError(int fldno, ISOException e) {
        appendErrorMessage("Invalid field; Field number = " + fldno + "; " + e.toString());
    }

    /**
     * Appends an error message for missing required vield.
     * @param fldno field number of the missing field.
     */
    private void appendFieldError(int fldno) {
        appendErrorMessage("Missing field; Field number = " + fldno);
    }

    /**
     * Sets a field in requestMsg.
     * @param fldno field number of the field to be set.
     * @param value to be set.
     */
    private void setField(int fldno, String value) {
        try {
            requestMsg.set(new ISOField(fldno, value));
            if (packager != null) {
                byte[] temp = packager.getFieldPackager(fldno).pack(requestMsg.getComponent(fldno));
                debugLogger.debug("id = " + fldno
                             + "; hex = " + ISOUtil.hexString(temp)
                             + "; description = " + packager.getFieldDescription(requestMsg, fldno));
            }
        } catch (ISOException e) {
            appendFieldError(fldno, e);
        }
    }

    /**
     * Sets a field in requestMsg.
     * @param fldno field number of the field to be set.
     * @param value to be set.
     */
    private void setField(int fldno, byte[] value) {
        try {
            requestMsg.set(new ISOBinaryField(fldno, value));
            if (packager != null) {
                byte[] temp = packager.getFieldPackager(fldno).pack(requestMsg.getComponent(fldno));
                debugLogger.debug("id = " + fldno
                             + "; hex = " + ISOUtil.hexString(temp)
                             + "; description = " + packager.getFieldDescription(requestMsg, fldno));
            }
        } catch (ISOException e) {
            appendFieldError(fldno, e);
        }
    }

    /**
     * Gets a field from the responseMsg.
     * @param fldno field number of the field to be retrieved.
     * @return fieldValue
     */
    private String getField(int fldno) {
        if (responseMsg.hasField(fldno)) {
            return responseMsg.getString(fldno);
        } else {
            return "";
        }
    }
    
    private static org.jpos.util.Logger getLogger() {
        return jPOSLogger;
    }

}