package com.commtrain.PMS.server;

import org.jpos.iso.*;

import com.commtrain.PMS.application.Application;
import com.commtrain.PMS.logging.Statistics;

/**
 * 
 */
public class ISORequestListenerImpl implements ISORequestListener {

    private static org.apache.log4j.Logger logger = org.apache.log4j.Logger.getLogger(ISORequestListenerImpl.class);
    private int maxWorkThreadsNo;
    private int activeWorkThreads=0;
    private Object lock = new Object();
    private Statistics statistics = new Statistics("Active working threads",100);

    /**
     * @param maxWorkThreadsNo Max number of concurently executed requests. 
     */
    public ISORequestListenerImpl(int maxWorkThreadsNo) {
        this.maxWorkThreadsNo=maxWorkThreadsNo;
    }

    public boolean process(ISOSource source, ISOMsg isoRequest) {

        WorkThread workThread;
        int currActiveWT;
        synchronized(lock){
            while(activeWorkThreads>=maxWorkThreadsNo){
                try {
                    lock.wait();
                } catch (InterruptedException e) { }
            };
            
            workThread = new WorkThread(isoRequest.toString(), source, isoRequest);
            currActiveWT = ++activeWorkThreads;
        }
        workThread.start();
        if (statistics.addEvent(currActiveWT)) {
            logger.info(statistics);
        }

        return true;
    }

    /**
     * Worker thread for terminal sessions
     * 
     */
    private class WorkThread extends Thread {
        private ISOMsg isoRequest;
        private ISOSource source;
        /**
         * WorkThread constructor
         * @param id session id
         * @param session terminal session
         */
        private WorkThread(String id, ISOSource source, ISOMsg isoRequest) {
            setName(id);
            this.source = source;
            this.isoRequest = isoRequest;
        }
        /**
         * @see java.lang.Runnable#run()
         */
        public void run() {

            try {
                Application app = Application.getInstance(isoRequest);

                if (app != null)
                    app.process(source, isoRequest);
            } catch (Throwable e) {
                logger.error("Stack Trace", e);
            }

            synchronized(lock){            
                activeWorkThreads--;
                lock.notify();
            }
        }
        
    }
}
