package com.commtrain.PMS.server;

import java.io.*;
import java.net.*;
import java.util.*;

import org.jpos.iso.*;
import org.jpos.util.*;

/**
 * 
 *
 */

public class Server extends Observable implements LogSource, Runnable, Observer /*, ISOServerMBean*/ {

    private static org.apache.log4j.Logger jPMSLogger = org.apache.log4j.Logger.getLogger(Server.class); //////NEW-------------------------

    int port;
    ISOChannel clientSideChannel;
    Class clientSideChannelClass;
    ISOPackager clientPackager;
    Collection clientOutgoingFilters, clientIncomingFilters, listeners;
 //   ThreadPool pool;
    public static final int DEFAULT_MAX_THREADS = 100;
    String name;
    protected Logger logger;
    protected String realm;
    protected ISOServerSocketFactory socketFactory = null;
    public static final int CONNECT = 0;
    public static final int SIZEOF_CNT = 1;
    private int[] cnt;
    private int activeSessions = 0; //Thread pool removed
    private int sessionId = 0; //Thread pool removed

    /**
     * @param port port to listen
     * @param clientSide client side ISOChannel (where we accept connections)
     * @param pool ThreadPool (created if null)
     */
    public Server(int port, ServerChannel clientSide /*, ThreadPool pool*/) {
        super();
        this.port = port;
        this.clientSideChannel = clientSide;
        this.clientSideChannelClass = clientSide.getClass();
        this.clientPackager = clientSide.getPackager();
        if (clientSide instanceof FilteredChannel) {
            FilteredChannel fc = (FilteredChannel) clientSide;
            this.clientOutgoingFilters = fc.getOutgoingFilters();
            this.clientIncomingFilters = fc.getIncomingFilters();
        }
        //Thread pool removed
//        this.pool = (pool == null) ? new ThreadPool(1, DEFAULT_MAX_THREADS) : pool;
        listeners = new Vector();
        name = "";
        cnt = new int[SIZEOF_CNT];
    }

    protected class Session implements Runnable, LogSource {
        ServerChannel channel;
        String realm;
        protected Session(ServerChannel channel) {
            this.channel = channel;
            realm = Server.this.getRealm() + ".session";
        }
        public void run() {
            if (channel instanceof BaseChannel) {
                Socket socket = ((BaseChannel)channel).getSocket ();
                realm = realm + socket.getInetAddress();
            }
            LogEvent logEvent = new LogEvent(this, "session-start"); //Thread pool removed
            
            int sessions = addSession();
            logEvent.addMessage(
                "active sessions",
                Integer.toString(sessions));
            Logger.log(logEvent);
            StartServer.createGUI(channel); //////NEW-------------------------
            try {
                for (;;) {
                    ISOMsg m = channel.receive();
                    Iterator iter = listeners.iterator();
                    while (iter.hasNext())
                        if (((ISORequestListener) iter.next()).process(channel, m))
                            break;
                }
            } catch (EOFException e) {
                Logger.log(new LogEvent(this, "session-warning", "<eof/>"));
                try {
                    channel.disconnect();
                } catch (IOException ex) {
                }
            } catch (InterruptedIOException e) {
                try {
                    channel.disconnect();
                } catch (IOException ex) {
                }
            } catch (Exception e) {
                LogEvent evt = new LogEvent(this, "session-error", e);
                try {
                    channel.disconnect();
                } catch (IOException ex) {
                    evt.addMessage(ex);
                }
                Logger.log(evt);
            }
            logEvent = new LogEvent(this, "session-end"); //Thread pool removed
            sessions = removeSession();
            logEvent.addMessage(
                "active sessions",
                Integer.toString(sessions));
            Logger.log (logEvent);
        }
        public void setLogger(Logger logger, String realm) {
        }
        public String getRealm() {
            return realm;
        }
        public Logger getLogger() {
            return Server.this.getLogger();
        }
    }
    /**
     * add an ISORequestListener
     * @param l request listener to be added
     * @see ISORequestListener
     */
    public void addISORequestListener(ISORequestListener l) {
        listeners.add(l);
    }
    /**
     * remove an ISORequestListener
     * @param l a request listener to be removed
     * @see ISORequestListener
     */
    public void removeISORequestListener(ISORequestListener l) {
        listeners.remove(l);
    }

    public void run() {
        ServerChannel channel;
        //Thread pool removed
//        if (!(pool == null)) {
//            pool.setLogger(this.getLogger(), this.getRealm() + ".pool");
//        }
        for (;;) {
            try {
            	ServerSocket serverSocket=null;
            	try{
	                serverSocket = socketFactory != null ? socketFactory.createServerSocket(port) : (new ServerSocket(port));
                } catch(BindException ex){
	            	jPMSLogger.error("Stack Trace", ex); //////NEW-------------------------
	            	return;
            	}
	            

                Logger.log(new LogEvent(this, "iso-server", "listening on port " + port));
                for (;;) {
                    try {
                        channel = (ServerChannel) clientSideChannelClass.newInstance();
                        channel.setPackager(clientPackager);
                        if (channel instanceof LogSource) {
                            ((LogSource) channel).setLogger(getLogger(), getRealm() + ".channel");
                        }
                        if (clientSideChannel instanceof BaseChannel) {
                            ((BaseChannel) channel).setHeader(((BaseChannel) clientSideChannel).getHeader());
                            ((BaseChannel) channel).setTimeout(((BaseChannel) clientSideChannel).getTimeout());
                        }
                        setFilters(channel);
                        if (channel instanceof Observable)
                             ((Observable) channel).addObserver(this);
                        jPMSLogger.info("Before accept");
                        channel.accept(serverSocket);
                        jPMSLogger.info("After accept");
                        cnt[CONNECT]++;
                        //Thread pool removed
                        sessionId++;
                        new Thread(
                            new Session(channel),
                            "session-"
                                + Integer.toString(sessionId)
                                + "-"
                                + ((BaseChannel)channel).getSocket().getInetAddress())
                            .start();
//                        pool.execute(new Session(channel));
                    } catch (IOException e) {
                        Logger.log(new LogEvent(this, "iso-server", e));
                        relax();
                    } catch (InstantiationException e) {
                        Logger.log(new LogEvent(this, "iso-server", e));
                        relax();
                    } catch (IllegalAccessException e) {
                        Logger.log(new LogEvent(this, "iso-server", e));
                        relax();
                    }
                }
            } catch (Throwable e) {
                Logger.log(new LogEvent(this, "iso-server", e));
                relax();
            }
        }
    }

    private void setFilters(ISOChannel channel) {
        if (clientOutgoingFilters != null)
             ((FilteredChannel) channel).setOutgoingFilters(clientOutgoingFilters);
        if (clientIncomingFilters != null)
             ((FilteredChannel) channel).setIncomingFilters(clientIncomingFilters);
    }
    private void relax() {
        try {
            Thread.sleep(5000);
        } catch (InterruptedException e) {
        }
    }

    /**
     * associates this Server with a name using NameRegistrar
     * @param name name to register
     * @see NameRegistrar
     */
    public void setName(String name) {
        this.name = name;
        NameRegistrar.register("server." + name, this);
    }
    /**
     * @return ISOMUX instance with given name.
     * @throws NameRegistrar.NotFoundException;
     * @see NameRegistrar
     */
    public static Server getServer(String name) throws NameRegistrar.NotFoundException {
        return (Server) NameRegistrar.get("server." + name);
    }
    /**
     * @return this Server's name ("" if no name was set)
     */
    public String getName() {
        return this.name;
    }
    public void setLogger(Logger logger, String realm) {
        this.logger = logger;
        this.realm = realm;
    }
    public String getRealm() {
        return realm;
    }
    public Logger getLogger() {
        return logger;
    }
    public void update(Observable o, Object arg) {
        setChanged();
        notifyObservers(arg);
    }
    /**
     * Gets the ISOClientSocketFactory (may be null)
     * @see     ISOClientSocketFactory
     * @since 1.3.3
     */
    public ISOServerSocketFactory getSocketFactory() {
        return socketFactory;
    }
    /**
     * Sets the specified Socket Factory to create sockets
     * @param         socketFactory the ISOClientSocketFactory
     * @see           ISOClientSocketFactory
     * @since 1.3.3
     */
    public void setSocketFactory(ISOServerSocketFactory socketFactory) {
        this.socketFactory = socketFactory;
    }
    public int getPort() {
        return port;
    }
    public void resetCounters() {
        cnt = new int[SIZEOF_CNT];
    }
    /**
     * @return number of connections accepted by this server
     */
    public int getConnectionCount() {
        return cnt[CONNECT];
    }

    //Thread pool removed
    // ThreadPoolMBean implementation (delegate calls to pool)
//    public int getJobCount() {
//        return pool.getJobCount();
//    }
//    public int getPoolSize() {
//        return pool.getPoolSize();
//    }
//    public int getMaxPoolSize() {
//        return pool.getMaxPoolSize();
//    }
//    public int getIdleCount() {
//        return pool.getIdleCount();
//    }
//    public int getPendingCount() {
//        return pool.getPendingCount();
//    }

    private synchronized int addSession() {
        return ++activeSessions;
    }
    private synchronized int removeSession() {
        return --activeSessions;
    }
}
