package com.commtrain.PMS.server;

import java.awt.*;
import java.io.*;
import java.text.SimpleDateFormat;
import java.util.Date;

import javax.swing.*;

import org.apache.log4j.BasicConfigurator;
import org.apache.log4j.ConsoleAppender;
import org.apache.log4j.PatternLayout;
import org.apache.log4j.Priority;
import org.apache.log4j.PropertyConfigurator;
import org.jpos.iso.*;
import org.jpos.iso.channel.*;
import org.jpos.iso.gui.*;
import org.jpos.util.Logger;

import com.commtrain.PMS.logging.Logger4J;
import com.commtrain.PMS.Props;
import com.commtrain.PMS.db.Connector;
import com.commtrain.PMS.iso.packager.*;

/**
 * , vangelov
 *
 * Start ISO Server. <br>
 * There is one required parameter <i>properties file name</i>. <br>
 */
public class StartServer {
	
	protected static JFrame frame;
    protected static Logger jPOSLogger = new Logger4J(Priority.INFO_INT);
    protected static boolean useUI;
    private static org.apache.log4j.Logger logger = org.apache.log4j.Logger.getLogger(StartServer.class);

    public static void main(String[] args) {

        if (args.length != 1) {
            printUsage();
            return;
        }

		try{
            Props.setFileName(args[0]);
            
            //Redirect out and err streams
            String logfilePrefix = Props.get("logFile.namePrefix", "ISOServer");
            try {
            	String logfileSuffix = Props.get("logFile.datePattern", "'.'yyyy-MM-dd");
                SimpleDateFormat formatter = new SimpleDateFormat(logfileSuffix);
                Date now = new Date();
                logfilePrefix += formatter.format(now);

                OutputStream outStream = new FileOutputStream(logfilePrefix + ".out", true);
                System.setOut(new PrintStream(outStream, true));
                System.out.println("\nISOServer starting " + now);

                OutputStream errStream = new FileOutputStream(logfilePrefix + ".err", true);
                System.setErr(new PrintStream(errStream, true));
            } catch (Exception e) {
                e.printStackTrace();
                return;
            }
            
            PropertyConfigurator.configureAndWatch(
                Props.getFilePath(),
                Props.getLong("log4jrefresh", 300000));
                
	        String jdbcDriver = Props.get("JDBCDriver");
	        String jdbcURL = Props.get("JDBCURL");
	        String jdbcUsername = Props.get("JDBCUsername");
	        String jdbcPassword = Props.get("JDBCPassword");
	        Connector.setJDBCParams(jdbcDriver, jdbcURL, jdbcUsername, jdbcPassword);
		}catch (Exception e) {
            e.printStackTrace();
            return;
        }

        ServerChannel serverChannel;
        int port;

        useUI = Props.getBoolean("useUI",false);

        try {
            port = Props.getInt("port");

	        byte[] TPDU = new byte[5];
			ISOPackager packager = new HypercomPackager();
			
			if(Props.getBoolean("logPackaging",false)){
				packager.setLogger(getLogger(), "HypercomPackager");
			}

            serverChannel = new NACChannel(packager, TPDU);

        } catch (Exception e) {
			logger.fatal("Stack Trace", e);
            return;
        }
        
        Server isoServer = new Server(port, serverChannel /*, null */); // Thread pool removed
        isoServer.setLogger(getLogger(), "ISOServer");
        int maxWorkThreadsNo = Props.getInt("maxWorkThreadsNo",100);        
        isoServer.addISORequestListener(new ISORequestListenerImpl(maxWorkThreadsNo));

        isoServer.run();

    }

    /**
     * Method printUsage.
     */
    private static void printUsage() {
        BasicConfigurator.configure(new ConsoleAppender(new PatternLayout("%m%n")));
        logger.error("Usage:\n\tStartServer <propertiesFileName>");
    }

    protected static void createGUI(ISOChannel channel) {

		if(!useUI)
			return;//Don't use User Interface
			
		if(frame==null){
	        frame = new JFrame("ISO Server");
/*        WindowListener l = new WindowAdapter() {
            public void windowClosing(WindowEvent e) {
                System.exit(0);
            }
        };
        frame.addWindowListener(l);*/

	        JOptionPane.setRootFrame(frame);
	        frame.getContentPane().setLayout(new GridLayout(0,1));
    	}


//        frame.getContentPane().add("Center",new ISOChannelPanel(channel, "Server GUI"));
        frame.getContentPane().add(new ISOChannelPanel(channel, "Listener "+channel.getName()));
        frame.pack();
        frame.show();

    }
    
    /**
     * Returns the logger.
     * @return Logger
     */
    private static Logger getLogger() {
        return jPOSLogger;
    }

}

