package com.commtrain.PMS.transaction;

import java.sql.SQLException;
import java.text.SimpleDateFormat;
import java.util.*;

import org.jpos.iso.*;

import com.commtrain.PMS.db.ProcsFuncs;

/**
 * 
 *
 */
public abstract class OfflineTransaction extends Transaction {

    private static org.apache.log4j.Logger logger = org.apache.log4j.Logger.getLogger(OfflineTransaction.class);

    /**
     * Constructor for OfflineTransaction.
     * @param isoRequest
     */
    public OfflineTransaction(ISOMsg isoRequest) {
        super(isoRequest);
    }


    abstract public void processOffline() throws ISOException, SQLException;


    /**
     * @see com.commtrain.PMS.transaction.Transaction#process()
     * @
     */
    public ISOMsg process() {
        try {

            // Prepare response
	        int mti = Integer.valueOf(isoRequest.getString(MESSAGE_TYPE_INDICATOR)).intValue() + 10;
	        isoResponse.set(MESSAGE_TYPE_INDICATOR, ISOUtil.padleft(Integer.toString(mti), 4, '0'));
	        isoResponse = copyCommonISOFields(isoRequest, isoResponse);

			Date dateFromRequest = getDate(TransactionUtils.getString(isoRequest, DATE_LOCAL_TRANSACTION), TransactionUtils.getString(isoRequest, TIME_LOCAL_TRANSACTION) );
			if(dateFromRequest!=null)
				startTransactionDate = dateFromRequest;
				
			TransactionUtils.setDateTimeInISOMsg(isoResponse, startTransactionDate);

	    	calcTransactionContext();
	    	isoResponse.set(CARD_ACCEPTOR_IDENTIFICATION_CODE, ISOUtil.zeropad(transactionContext.getVUNumber(), 15));

			processOffline();

	
            return isoResponse;
        } catch (Exception e) {
       		logger.error("Stack Trace", e);

            if (status == STATUS_RQ_RECEIVED) {
                ProcsFuncs.addFakeCDSRecord(isoRequest);
            }

	        String retrievalReferenceNum = TransactionUtils.getString(isoResponse, RETRIEVAL_REFERENCE_NUM, "");
	        String responseCode = TransactionUtils.getString(isoResponse, RESPONSE_CODE, RESPONSE_CODE_REENTER_TRANSACTION);
	        return createErrorMessage(isoRequest, retrievalReferenceNum, null, responseCode);
        }
    }
    
    /**
     * Method getDate retrieves closest Date object, which could be coded in param Strings.
     * @param dateRepresentation in pattern "MMdd"
     * @param timeRepresentation in pattern "HHmmss"
     * @return Date
     */
    public static Date getDate(String dateRepresentation, String timeRepresentation){
    	Date res=null;
    	Date now = new Date();
    	
    	if(dateRepresentation == null || timeRepresentation == null)
    		return null;
    		
    	try{
    		GregorianCalendar calendar = new GregorianCalendar();
    		calendar.setTime(now);
    		int curYear = calendar.get(Calendar.YEAR);
    		
    		calendar.setTime(new SimpleDateFormat("MMdd:HHmmss").parse(dateRepresentation+':'+timeRepresentation));
    		
			calendar.set(Calendar.YEAR, curYear-1);
			res = calendar.getTime();

			calendar.set(Calendar.YEAR, curYear);
			if(Math.abs(res.getTime()- now.getTime() )  >   Math.abs(calendar.getTime().getTime() - now.getTime()) ){
				res=calendar.getTime();
			}

			calendar.set(Calendar.YEAR, curYear+1);
			if(Math.abs(res.getTime()- now.getTime() )  >   Math.abs(calendar.getTime().getTime() - now.getTime()) ){
				res=calendar.getTime();
			}
    	}catch (Exception e) {
            return null;
        }
    	
    	return res;
    }
    
}
