package com.commtrain.PMS.transaction;

import java.lang.reflect.Constructor;
import java.lang.reflect.InvocationTargetException;
import java.math.BigDecimal;
import java.sql.SQLException;


import org.jpos.iso.*;

import com.commtrain.PMS.application.Application;
import com.commtrain.PMS.db.*;


/**
 * 
 * Abstract base class for all transaction implementation classes.
 */
public abstract class Transaction {

    private static org.apache.log4j.Logger logger = org.apache.log4j.Logger.getLogger(Transaction.class);

    protected ISOMsg isoRequest;
    protected ISOMsg isoResponse;
    protected int status;
    protected java.util.Date startTransactionDate = new java.util.Date();


    public static final String POS_LOCAL_CURRENCY = "756";
    protected static final int STATUS_RQ_RECEIVED = 0;
    protected static final int STATUS_RQ_REGISTERED = 10;
    protected static final int STATUS_RQ_ERROR = 20;
    protected static final int STATUS_RQ_CONVERTED = 20;
    protected static final int STATUS_RQ_ONLINE_AUTHORISED = 30;
    protected static final int STATUS_RQ_ONLINE_RESERVATION = 34;
    protected static final int STATUS_RQ_ONLINE_NOT_AUTHORISED = 40;
    protected static final int STATUS_REQUEST_FOR_STTLEMENT = 50;
    protected static final int STATUS_NOT_FOR_SETTLEMENT = 100;
    protected static final int STATUS_TRANSACTION_VOID = 101;
    protected static final int STATUS_TRANSACTION_REVERSAL = 101;
    protected static final int STATUS_TRANSACTION_PAPER_ORDER = 130;
    
    protected static final int STATUS_DETAIL_NORMAL_CREDIT = 0;
    protected static final int STATUS_DETAIL_VOID = 1;
    protected static final int STATUS_DETAIL_DEBIT = 2;
    protected static final int STATUS_DETAIL_REFERRAL = 3;
    protected static final int STATUS_DETAIL_RESERVATION = 4;
    protected static final int STATUS_DETAIL_TIP = 5;
    protected static final int STATUS_DETAIL_CASH_ADVICE = 6;
    protected static final int STATUS_DETAIL_MOTO = 7;
    protected static final int STATUS_DETAIL_RESERVATION_CAPTURE = 8;

    public static final int MESSAGE_TYPE_INDICATOR = 0;
	public static final int PAN_PRIMARY_ACC_NUM = 2;
	public static final int PROCESSING_CODE = 3;
	public static final int AMOUNT_TRANSACTION = 4;
	public static final int SYSTEM_TRACE_AUDIT_NUM = 11;
	public static final int TIME_LOCAL_TRANSACTION = 12;
	public static final int DATE_LOCAL_TRANSACTION = 13;
	public static final int DATE_EXPIRATION = 14;
	public static final int POS_ENTRY_MODE = 22;
	public static final int NET_INTERNATIONAL_IDENTIFIEER = 24;
	public static final int POS_CONDITION_CODE = 25;
	public static final int TRACK_2_DATA = 35;
	public static final int RETRIEVAL_REFERENCE_NUM = 37;
	public static final int AUTHORIZATION_IDENTIFICATION_RESPONSE = 38;
	public static final int RESPONSE_CODE = 39;
	public static final int TERMINAL_IDENTIFICATION = 41;
	public static final int CARD_ACCEPTOR_IDENTIFICATION_CODE = 42;
	public static final int ADDITIONAL_RESPONSE_DATA = 44;
	public static final int TRACK_1_DATA = 45;
    public static final int CURRENCY_CODE_TRANSACTION = 49;
	public static final int SECURITY_CONTROL_INFO = 53;
	public static final int ADDITIONAL_AMOUNTS = 54;
	public static final int BATCH_NO = 60;
	public static final int ORIGINAL_AMOUNT = 60;
	public static final int DISCOUNT = 61;
	public static final int INVOICE_ECR_REFNO = 62;
    public static final int RECONCILIATION_REQUEST_TOTALS = 63;
	public static final int ADDITIONAL_DATA = 63;


    public static final String RESPONSE_CODE_APPROVAL = "00";
    public static final String RESPONSE_CODE_PLEASE_CALL = "01";
  	public static final String RESPONSE_CODE_REFERRAL = "02";
    public static final String RESPONSE_CODE_NOT_ACCEPTED = "05";
    public static final String RESPONSE_CODE_ACCEPTED = "09";
	public static final String RESPONSE_CODE_REENTER_TRANSACTION = "19";
	public static final String RESPONSE_CODE_DECLINED = "51";
    public static final String RESPONSE_CODE_CARD_EXPIRED = "54";
	public static final String RESPONSE_CODE_INVALID_TRANSACTION = "58";
    public static final String RESPONSE_CODE_TRANSACTION_NOT_FOUND = "78";
    public static final String RESPONSE_CODE_BAD_TERMINAL_ID = "89";

    protected static final String POS_CONDITION_CODE_NORMAL_PRESENTMENT = "00";
    protected static final String POS_CONDITION_CODE_PREAUTHORIZED_REQUEST = "06";
    protected static final String POS_CONDITION_CODE_MAIL_PHONE_ORDER = "08";
    protected static final String POS_CONDITION_CODE_UNSECURE = "16";
    protected static final String POS_CONDITION_CODE_SECURE = "18";
    protected static final int CAR_TERMINAL_TYPE_NORMAL = 2100;
    protected static final int CAR_TERMINAL_TYPE_INTERNET_UNSECURE = 2500;
    protected static final int CAR_TERMINAL_TYPE_INTERNET_SECURE = 2510;


    protected BigDecimal cdsID; // TransactionContext now knows cdsID
	protected TransactionContext transactionContext;

    /**
     * Contructor Transaction.
     * @param isoRequest
     */
    protected Transaction(ISOMsg isoRequest){
        this.isoRequest = isoRequest;
        this.isoResponse = new ISOMsg();
        this.status = STATUS_RQ_RECEIVED;
    }

    public abstract ISOMsg process();

    public static Transaction getInstance(Application application, ISOMsg isoRequest) {
        Transaction transaction = null;

/*    	byte[] headerBytes = isoRequest.getHeader();
    	if(headerBytes==null)
    		return transaction;
    	IENHeader header = new IENHeader();
    	header.unpack(headerBytes);
        String clientType = Byte.toString(header.getId());*/
        String applicationID = Integer.toString(application.getID());
        String mti = TransactionUtils.getString(isoRequest, MESSAGE_TYPE_INDICATOR, "");
        String processingCode = TransactionUtils.getString(isoRequest, PROCESSING_CODE, "");
        String transactionClassName = TransactionMapping.getImplementationClassname(applicationID, mti, processingCode);
        if(transactionClassName!=null){
	        try {
	            Class classInst = Class.forName(transactionClassName);
	            Class[] parameters = { ISOMsg.class };
	            Constructor constructor = classInst.getConstructor(parameters);
	            transaction = (Transaction) constructor.newInstance(new Object[] { isoRequest });
	        } catch (NoSuchMethodException e) {
	        } catch (InvocationTargetException e) {
	        } catch (InstantiationException e) {
	        } catch (IllegalAccessException e) {
	        } catch (ClassNotFoundException e) {
	        }
        }
        return transaction;
    }

    /**
     * Method getCardDetails.
     */
	protected String getCardDetails() throws ISOException{
        return TransactionUtils.getCardDetails(isoRequest, TransactionUtils.CARD_DETAILS_DEPEND_ON_FLAG);
    }

    public static ISOMsg createErrorMessage(ISOMsg isoRequest, String retrievalReferenceNumber, String approvalCode, String responseCode) {
        ISOMsg errMsg = new ISOMsg();
        try {
            errMsg.setMTI( isoRequest.getString(MESSAGE_TYPE_INDICATOR));
            errMsg.setResponseMTI();
            errMsg.set(PROCESSING_CODE, TransactionUtils.getString(isoRequest, PROCESSING_CODE, ""));
            errMsg.set(SYSTEM_TRACE_AUDIT_NUM, TransactionUtils.getString(isoRequest, SYSTEM_TRACE_AUDIT_NUM, ""));
            TransactionUtils.setDateTimeInISOMsg(errMsg, new java.util.Date());

            errMsg.set(NET_INTERNATIONAL_IDENTIFIEER, TransactionUtils.getString(isoRequest, NET_INTERNATIONAL_IDENTIFIEER, ""));
            errMsg.set(RETRIEVAL_REFERENCE_NUM, ISOUtil.strpad(retrievalReferenceNumber,12) );
            if (approvalCode != null) {
                errMsg.set(AUTHORIZATION_IDENTIFICATION_RESPONSE, approvalCode);
            }
            errMsg.set(RESPONSE_CODE, ISOUtil.strpad(responseCode,2));
            errMsg.set(TERMINAL_IDENTIFICATION, ISOUtil.strpad(TransactionUtils.getString(isoRequest, TERMINAL_IDENTIFICATION, ""), 8));
        } catch (Exception e) {
            logger.error("Unexpected error " + e);
        }
        return errMsg;
    }
    
    /**
     * Uses the isoRequest to distinguish the Transaction code (CD_1090).
     * @return int
     * @deprecated use TransactionUtils.getTransactionCode(isoRequest)
     */
    protected int transactionCode() {
		return TransactionUtils.getTransactionCode(isoRequest);
	}

   
    /**
     * Copies PROCESSING_CODE, SYSTEM_TRACE_AUDIT_NUM, NET_INTERNATIONAL_IDENTIFIEER, TERMINAL_IDENTIFICATION from the ISORequest to ISOResponse.
     * @param request source
     * @param response destination
     * @return CARRequest
     */
    protected static ISOMsg copyCommonISOFields(ISOMsg request, ISOMsg response) throws ISOException{
        response.set(PROCESSING_CODE, request.getString(PROCESSING_CODE));
        response.set(SYSTEM_TRACE_AUDIT_NUM, request.getString(SYSTEM_TRACE_AUDIT_NUM));
        response.set(NET_INTERNATIONAL_IDENTIFIEER, request.getString(NET_INTERNATIONAL_IDENTIFIEER));
        response.set(TERMINAL_IDENTIFICATION, request.getString(TERMINAL_IDENTIFICATION));
        return response;
    }


    protected void calcTransactionContext()  throws SQLException, ISOException {
    	calcTransactionContext(getCardDetails(), isoRequest.getString(TERMINAL_IDENTIFICATION));
    }

    protected void calcTransactionContext(String cardDetails, String terminalID) throws SQLException {
		this.transactionContext = new TransactionContext(cardDetails, terminalID);
    }

    /**
     * @return true when license is valid and transaction should be performed
     */
    protected boolean isLicenseValid(){
        if(transactionContext == null )
            return false;
        return true;
    }
}
