package com.commtrain.PMS.transaction.hypercom;

import java.math.*;
import java.sql.*;

import org.jpos.iso.ISOException;
import org.jpos.iso.ISOMsg;
import org.jpos.iso.ISOUtil;

import com.commtrain.PMS.db.*;
import com.commtrain.PMS.iso.*;
import com.commtrain.PMS.transaction.*;

public class OnlineSale extends OnlineSaleBase {

    protected TransactionRegister transactionRegister;
    
    public OnlineSale(ISOMsg isoRequest) {
        super(isoRequest);
    }

    /**
     * @see OnlineTransaction#convertResponse()
     */
    public void convertResponse() throws ISOException, SQLException {
        int mti = Integer.valueOf(isoRequest.getString(MESSAGE_TYPE_INDICATOR)).intValue() + 10;

        String carResponseIndicator = acqResponse.getString(CARResponse.RESPONSE_INDICATOR);
        String carAddDataResponse = TransactionUtils.getString(acqResponse, CARResponse.ADDITIONAL_DATA);
        String carResponseCode = getCustomCarResponseCode(acqResponse.getString(CARResponse.RESPONSE_CODE) , carResponseIndicator);
		
		
        if (CARResponse.RESPONSE_CODE_TRANSACTION_ACCEPTED.equals(carResponseCode) && CARResponse.RESPONSE_INDICATOR_APPROVAL.equals(carResponseIndicator) ) {
            status = STATUS_RQ_ONLINE_AUTHORISED + this.getTransactionKind();
            isoResponse.set(ADDITIONAL_DATA,
                            Messages.getStringSignaturePlease(transactionContext.getLocaleDescription()).getBytes());
        } else if(CARResponse.RESPONSE_INDICATOR_INCREASE_TO.equals(carResponseIndicator) ){
            status = STATUS_RQ_CONVERTED;
        }else{
        	status = STATUS_RQ_ONLINE_NOT_AUTHORISED; // Additional Data will be send only in case of carResponseCode == 59 or 60!!! (see code below!)
        	if((CARResponse.RESPONSE_CODE_CALL_REFERRAL1.equals(carResponseCode)||CARResponse.RESPONSE_CODE_CALL_REFERRAL2.equals(carResponseCode)) && carResponseIndicator.equals(CARResponse.RESPONSE_INDICATOR_REFERAL)){
				if ((carAddDataResponse == null)||("".equals(carAddDataResponse.trim()))){
					carAddDataResponse = transactionContext.getAcqHelpdeskNo();
				}
	       	    isoResponse.set(ADDITIONAL_DATA,
	       	     (Messages.getStringReferral(transactionContext.getLocaleDescription()) + " " + carAddDataResponse).getBytes());
	        	status += STATUS_DETAIL_REFERRAL;
        	}
        }

        isoResponse.set(MESSAGE_TYPE_INDICATOR, ISOUtil.padleft(Integer.toString(mti), 4, '0'));
        TransactionUtils.setDateTimeInISOMsg(isoResponse, startTransactionDate);

    	isoResponse.set(CARD_ACCEPTOR_IDENTIFICATION_CODE, ISOUtil.zeropad(transactionContext.getVUNumber(), 15));

        // PROCESSING_CODE
        // SYSTEM_TRACE_AUDIT_NUM
        // NET_INTERNATIONAL_IDENTIFIEER
        // TERMINAL_IDENTIFICATION
        isoResponse = copyCommonISOFields(isoRequest, isoResponse);

        isoResponse.set(RETRIEVAL_REFERENCE_NUM, transactionRegister.getRetrivalRefNo());
        isoResponse.set(AUTHORIZATION_IDENTIFICATION_RESPONSE, acqResponse.getString(CARResponse.APPROVAL_CODE));
        isoResponse.set(RESPONSE_CODE, convertCar2IsoResponseCode(carResponseCode));


		Connection conn = null;
		try{//Start DB Transaction
			conn = Connector.getConnection();
			conn.setAutoCommit(false);
		
	        ProcsFuncs.updCDS_RP(conn, cdsID, Integer.valueOf(carResponseIndicator), carResponseCode, acqResponse.getString(CARResponse.APPROVAL_CODE), acqResponse.getFloorLimit(), status);
			if(status==STATUS_RQ_ONLINE_AUTHORISED + this.getTransactionKind()){
		        ProcsFuncs.addCSRecord(conn, transactionRegister.getTrmID(), TransactionUtils.getTransactionCode(isoRequest), status,  new BigDecimal(isoRequest.getString(AMOUNT_TRANSACTION)), transactionRegister.getCdsID());
			}

			conn.commit();
		}finally{//End DB Transaction
			if(conn!=null){
				conn.rollback();
				conn.setAutoCommit(true);
				Connector.releaseConnection(conn);
			}
		}


    }

    /**
     * @see OnlineTransaction#convertRequest()
     */
    public void convertRequest() throws ISOException, SQLException {
        int statusSuccess = STATUS_RQ_CONVERTED + getTransactionKind();
        int statusFailure = STATUS_RQ_REGISTERED + getTransactionKind();

        String dataCaptureIndicator = TransactionUtils.getDataCaptureIndicator(isoRequest);
        int carRecordType = dataCaptureIndicator.equals(TransactionUtils.CDS_ENTRY_MODE_AUTO) ? CARRequest.CAR_RECORD_TYPE_SALE_MAG : CARRequest.CAR_RECORD_TYPE_SALE_KBD;	

        transactionRegister = new TransactionRegister(isoRequest, carRecordType, startTransactionDate, statusSuccess, statusFailure);
        cdsID = transactionRegister.getCdsID();

        status = transactionRegister.getStatus();
        if (status == statusSuccess) {
            //Create CAR Request
            CARRequest req = new CARRequest();

            req = fillCommonCARFields(req, transactionContext, transactionRegister.getCarSeqNo());
            req.setField(CARRequest.TERMINAL_TYPE, Integer.toString(TransactionUtils.getCARTerminalType(isoRequest)));
            req.setField(CARRequest.RECORD_TYPE, Integer.toString(carRecordType));
            req.setField(CARRequest.CARD_DETAILS, TransactionUtils.getCardDetails(isoRequest, TransactionUtils.CARD_DETAILS_DEPEND_ON_FLAG));
            req.setField(CARRequest.AMOUNT, isoRequest.getString(AMOUNT_TRANSACTION));
            req.setField(CARRequest.CURRENCY, TransactionUtils.getCurrencyCode(isoRequest));
            acqRequest = req;

//          In case of later error, it will be returned to the terminal and printed in the log
            isoResponse.set(RETRIEVAL_REFERENCE_NUM, transactionRegister.getRetrivalRefNo());  
        } else
            throw new SQLException("OnlineSale transaction failure Registered!");
            
    }
}