package com.commtrain.PMS.transaction.hypercom;

import java.math.BigDecimal;
import java.sql.CallableStatement;
import java.sql.Connection;
import java.sql.SQLException;
import java.sql.Types;
import java.text.SimpleDateFormat;

import org.jpos.iso.ISOException;
import org.jpos.iso.ISOMsg;

import com.commtrain.PMS.db.Connector;
import com.commtrain.PMS.iso.CARRequest;
import com.commtrain.PMS.iso.CARResponse;
import com.commtrain.PMS.transaction.OfflineTransaction;
import com.commtrain.PMS.transaction.Transaction;
import com.commtrain.PMS.transaction.TransactionUtils;

/**
 * 
 */
public class ReservationCapture extends OfflineTransaction {

    private static org.apache.log4j.Logger logger = org.apache.log4j.Logger.getLogger(ReservationCapture.class);

    /**
     * Constructor for ReservationCapture.
     * @param isoRequest
     */

    public ReservationCapture(ISOMsg isoRequest) {
        super(isoRequest);
    }

    public void processOffline() throws ISOException, SQLException {

        final int STATUS_AUTHORISED = STATUS_RQ_ONLINE_AUTHORISED + STATUS_DETAIL_RESERVATION_CAPTURE;        
        final int STATUS_REGISTERED = STATUS_RQ_REGISTERED + STATUS_DETAIL_RESERVATION_CAPTURE;

        String trmID = TransactionUtils.getString(isoRequest, TERMINAL_IDENTIFICATION, "");
        String invoiceNo = TransactionUtils.getString(isoRequest, INVOICE_ECR_REFNO, "");
        String sysTrace = TransactionUtils.getString(isoRequest, SYSTEM_TRACE_AUDIT_NUM, "");
        String retrievalReferenceNo = TransactionUtils.getString(isoRequest, RETRIEVAL_REFERENCE_NUM, "");
//        isoResponse.set(RETRIEVAL_REFERENCE_NUM, retrievalReferenceNo);
        BigDecimal amount = new BigDecimal(TransactionUtils.getString(isoRequest, AMOUNT_TRANSACTION));
        int currencyCode = TransactionUtils.getIntCurrencyCode(isoRequest);
        String dataCaptureIndicator = TransactionUtils.getDataCaptureIndicator(isoRequest);
        String cardDetails = getCardDetails();
        int transactionCode = TransactionUtils.getTransactionCode(isoRequest);
        int carRecordType = dataCaptureIndicator.equals(TransactionUtils.CDS_ENTRY_MODE_AUTO) ? CARRequest.CAR_RECORD_TYPE_SALE_MAG : CARRequest.CAR_RECORD_TYPE_SALE_KBD;


        Connection conn = null;
        CallableStatement cstmt = null;

        //Start DB processing
        try {
            conn = Connector.getConnection();
            cstmt = conn.prepareCall("{call RESERVATION_CAPTURE(?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?)}");

//          terminalid IN VARCHAR2,
            cstmt.setString(1, trmID);

//          invoiceno IN VARCHAR2,
            cstmt.setString(2, invoiceNo);


//          systrace IN varchar2,
            cstmt.setString(3, sysTrace);

//          amount
            cstmt.setBigDecimal(4, amount);

//          tcdate
            cstmt.setInt(5, Integer.parseInt(new SimpleDateFormat("yyyyMMdd").format(startTransactionDate)));
            
//          tctime
            cstmt.setInt(6, Integer.parseInt(new SimpleDateFormat("HHmm").format(startTransactionDate)));

//          dataCaptureIndicator
            cstmt.setString(7, dataCaptureIndicator);

//          track2data
            cstmt.setString(8, cardDetails);

//          transaction_code
            cstmt.setInt(9, transactionCode);

//          trm_type
            cstmt.setInt(10, TransactionUtils.getCARTerminalType(isoRequest));

//          record_type
            cstmt.setInt(11, carRecordType);

//          approval_code
            cstmt.setString(12, CARResponse.APPROVAL_CODE_DEFAULT);

//          currency_code
            cstmt.setInt(13, currencyCode);
            
//          reserve_retrieval_no
            cstmt.setString(14, retrievalReferenceNo);
            
//          status_registered
            cstmt.setInt(15, STATUS_REGISTERED);
            
//          status_received IN
            cstmt.setInt(16, STATUS_RQ_RECEIVED);
            
//          status_authorised IN
            cstmt.setInt(17, STATUS_AUTHORISED);
            
//          status_declined
            cstmt.setInt(18, STATUS_RQ_ONLINE_NOT_AUTHORISED);
            
//          status_captured
            cstmt.setInt(19, STATUS_NOT_FOR_SETTLEMENT + STATUS_DETAIL_RESERVATION);
            
//          status
            cstmt.registerOutParameter(20, Types.INTEGER);
            
//          cdsid OUT number,
            cstmt.registerOutParameter(21, Types.NUMERIC);

//          retrieval_ref_no OUT varchar2,
            cstmt.registerOutParameter(22, Types.VARCHAR);

//          approval_code_reservation
            cstmt.registerOutParameter(23, Types.VARCHAR);

//          error_code
            cstmt.registerOutParameter(24, Types.INTEGER);

//          error_text
            cstmt.registerOutParameter(25, Types.VARCHAR);

            cstmt.execute();

            String sqlError = cstmt.getString(25);
            if (sqlError!=null) {
                logger.error(sqlError);
            }
            
            status = cstmt.getInt(20);

            if (status != STATUS_RQ_RECEIVED) {

                cdsID = cstmt.getBigDecimal(21);
                retrievalReferenceNo = cstmt.getString(22);
                isoResponse.set(RETRIEVAL_REFERENCE_NUM, retrievalReferenceNo);

                int errorCode = cstmt.getInt(24);
                String errorText;
                switch (errorCode) {
                    case 1 :
                        errorText = "Registration error.";
                        break;

                    case 2 :
                        errorText = "Ivalid amount.";
                        break;

                    case 3 :
                        errorText = "Invalid currency.";
                        break;

                    case 4 :
                        errorText = "Invalid PAN.";
                        break;

                    case 5 :
                        errorText = "Invalid invoice number.";
                        break;

                    case 6 :
                        errorText = "Other search error.";
                        break;

                    case 7 :
                        errorText = "Update error.";
                        break;

                    default :
                        errorText = "";
                        break;
                }
                if (errorText.length()>0) {
                    logger.info(errorText);
                }
                
                switch (status) {
                    case STATUS_AUTHORISED :
                        isoResponse.set(RESPONSE_CODE, RESPONSE_CODE_APPROVAL);
                        String approvalCodeISO = cstmt.getString(23);
                        if (approvalCodeISO != null) {
                            isoResponse.set(Transaction.AUTHORIZATION_IDENTIFICATION_RESPONSE, approvalCodeISO);
                        }
                        isoResponse.set(ADDITIONAL_DATA, Messages.getStringSignaturePlease(this.transactionContext.getLocaleDescription()).getBytes()); //$NON-NLS-1$
                        break;

                    case STATUS_RQ_ONLINE_NOT_AUTHORISED :
                        isoResponse.set(RESPONSE_CODE, RESPONSE_CODE_DECLINED);
                        isoResponse.set(AUTHORIZATION_IDENTIFICATION_RESPONSE, CARResponse.APPROVAL_CODE_DEFAULT);
                        break;

                    default :
                        isoResponse.set(RESPONSE_CODE, RESPONSE_CODE_INVALID_TRANSACTION);
                        isoResponse.set(AUTHORIZATION_IDENTIFICATION_RESPONSE, CARResponse.APPROVAL_CODE_DEFAULT);
                        break;
                }
            } else {
                isoResponse.set(RESPONSE_CODE, RESPONSE_CODE_INVALID_TRANSACTION);
            }
            
        } catch (SQLException e) {
       		logger.error("Stack Trace", e);
            isoResponse = createErrorMessage(isoRequest, retrievalReferenceNo, null, RESPONSE_CODE_INVALID_TRANSACTION);
            
        } finally {
            if (cstmt != null) {
                try {
                    cstmt.close();
                } catch (SQLException e) {
                    logger.warn(e);
                }
            }
            if (conn != null) {
                Connector.releaseConnection(conn);
            }
        }
    }

}