package com.commtrain.PMS.transaction.hypercom;

import java.io.IOException;
import java.math.BigDecimal;
import java.sql.SQLException;

import org.jpos.iso.*;

import com.commtrain.PMS.db.*;
import com.commtrain.PMS.iso.*;
import com.commtrain.PMS.transaction.*;

/**
 * 
 * Void Transaction. 
 */
public class Void extends OnlineTransaction {
	protected boolean needsAuthrization = false;
	private TransactionRegister transactionRegister;
    /**
     * Constructor for VoidTransaction.
     * @param isoRequest
     */
    public Void(ISOMsg isoRequest) {
        super(isoRequest);
    }


    public void convertRequest() throws ISOException, SQLException {
    	customConvertRequest(false);
    }


    public void customConvertRequest(boolean ignoreSQLExceptions) throws ISOException, SQLException {
        //            String dataCaptureIndicator = TransactionUtils.getDataCaptureIndicator(isoRequest);
        int carRecordType = CARRequest.CAR_RECORD_TYPE_REVERSAL;

        transactionRegister = new TransactionRegister(
			isoRequest.getString(Transaction.TERMINAL_IDENTIFICATION),
	    	isoRequest.getString(Transaction.INVOICE_ECR_REFNO), 
    		isoRequest.getString(Transaction.SYSTEM_TRACE_AUDIT_NUM), 
	    	new BigDecimal(isoRequest.getString(Transaction.AMOUNT_TRANSACTION)),
			startTransactionDate,
	    	TransactionUtils.getDataCaptureIndicator(isoRequest), 
	    	getCardDetails(),
	    	TransactionUtils.getTransactionCodeInverse(isoRequest),  //Stupid recommendation from certifiers to change code from 5 to 6 and vice versa
	    	TransactionUtils.getCARTerminalType(isoRequest),
	    	carRecordType,
	    	TransactionRegister.AUTHORIZATION_NO_DEFAULT,
            TransactionUtils.getIntCurrencyCode(isoRequest), 
	        STATUS_RQ_CONVERTED,
	        STATUS_RQ_REGISTERED);


        status = transactionRegister.getStatus();
        cdsID = transactionRegister.getCdsID();

        int oldStatus =  ProcsFuncs.voidCSRecord(transactionRegister.getTrmID(),
                                                 isoRequest.getString(Transaction.INVOICE_ECR_REFNO),
                                                 TransactionUtils.getTransactionCode(isoRequest),
                                                 new BigDecimal(isoRequest.getString(AMOUNT_TRANSACTION)));
        
        if (oldStatus == 0 && !ignoreSQLExceptions) {
            status = STATUS_RQ_REGISTERED;
            //        	throw new SQLException("Void failed, because original transaction is not found!");
        }
        
        if (status == STATUS_RQ_CONVERTED) {
        	needsAuthrization = (oldStatus != STATUS_RQ_ONLINE_AUTHORISED + STATUS_DETAIL_RESERVATION_CAPTURE)
        	                    && (oldStatus != STATUS_RQ_ONLINE_AUTHORISED + STATUS_DETAIL_DEBIT);
            if (needsAuthrization) {
	            // Create CAR Request
	            // Sets MANIFACTURE_ID, TERMINAL_ID, SEQUENCE_NO_ID, VU_NUMBER
	            CARRequest req = new CARRequest();
	
	            req = fillCommonCARFields(req, transactionContext, transactionRegister.getCarSeqNo());
	            req.setField(CARRequest.TERMINAL_TYPE, Integer.toString(TransactionUtils.getCARTerminalType(isoRequest)));
	            req.setField(CARRequest.RECORD_TYPE, Integer.toString(carRecordType));
	            req.setField(CARRequest.CARD_DETAILS, TransactionUtils.getCardDetails(isoRequest, TransactionUtils.CARD_DETAILS_WHICH_PRESENTS));
	            req.setField(CARRequest.AMOUNT, isoRequest.getString(AMOUNT_TRANSACTION));
	            req.setField(CARRequest.CURRENCY, TransactionUtils.getCurrencyCode(isoRequest));
	            acqRequest = req;
            }
        } else{
//            throw new SQLException("Reversal transaction failure Registered!");
	        status = STATUS_RQ_REGISTERED;
        }

//      In case of later error, it will be returned to the terminal and printed in the log
        isoResponse.set(RETRIEVAL_REFERENCE_NUM, transactionRegister.getRetrivalRefNo());  
    }


    /**
     * @see OnlineTransaction#convertResponse()
     */
    public void convertResponse() throws ISOException, SQLException {
        int mti = Integer.valueOf(isoRequest.getString(MESSAGE_TYPE_INDICATOR)).intValue() + 10;

        status = STATUS_TRANSACTION_REVERSAL;

        isoResponse.set(MESSAGE_TYPE_INDICATOR, ISOUtil.padleft(Integer.toString(mti), 4, '0'));
        TransactionUtils.setDateTimeInISOMsg(isoResponse, startTransactionDate);
        isoResponse = copyCommonISOFields(isoRequest, isoResponse);
    	isoResponse.set(CARD_ACCEPTOR_IDENTIFICATION_CODE, ISOUtil.zeropad(transactionContext.getVUNumber(), 15));

        String carApprovalCode = acqResponse.getString(CARResponse.APPROVAL_CODE);
        String carResponseCode = acqResponse.getString(CARResponse.RESPONSE_CODE);

        isoResponse.set(RETRIEVAL_REFERENCE_NUM, transactionRegister.getRetrivalRefNo());
        isoResponse.set(AUTHORIZATION_IDENTIFICATION_RESPONSE, carApprovalCode);
        isoResponse.set(RESPONSE_CODE, convertCar2IsoResponseCode(carResponseCode));

        // Register ISO/CAR Response
        // ... CALL DB_PROCEDURE.
        ProcsFuncs.updCDS_RP(transactionRegister.getCdsID(), Integer.valueOf(acqResponse.getString(CARResponse.RESPONSE_INDICATOR)), carResponseCode, carApprovalCode, acqResponse.getFloorLimit(), status);

    }

    /**
     * @see com.commtrain.PMS.transaction.OnlineTransaction#recieveAcqResponse()
     */
    protected CARResponse recieveAcqResponse() throws ISOException, IOException {
        // Prepare default CAR Response
        acqResponse = new CARResponse();
        acqResponse.set(CARResponse.RESPONSE_INDICATOR, CARResponse.RESPONSE_INDICATOR_APPROVAL);
        acqResponse.set(CARResponse.RESPONSE_CODE, CARResponse.RESPONSE_CODE_TRANSACTION_ACCEPTED);
        acqResponse.set(CARResponse.APPROVAL_CODE, CARResponse.APPROVAL_CODE_DEFAULT);
        acqResponse.setFloorLimit(new Integer(CARResponse.FLOOR_LIMIT_DEFAULT));
        return acqResponse;
    }



    /**
     * @see com.commtrain.PMS.transaction.Transaction#getCardDetails()
     */
    protected String getCardDetails() throws ISOException {
        return TransactionUtils.getCardDetails(isoRequest, TransactionUtils.CARD_DETAILS_MANUAL);
    }

    /**
     * @see com.commtrain.PMS.transaction.OnlineTransaction#setupAcquierCommunication()
     */
    protected void setupAcquierCommunication() {
        if (needsAuthrization) {
            super.setupAcquierCommunication();
        }
    }

    /**
     * @see com.commtrain.PMS.transaction.OnlineTransaction#sendAcqRequest(ISOMsg)
     */
    protected void sendAcqRequest(ISOMsg reqMsg) throws SQLException, ISOException, IOException {
        if (needsAuthrization) {
            super.sendAcqRequest(reqMsg);
        }
    }

}
