package com.commtrain.sql;
import java.sql.*;
import java.util.*;

public class SQLHelper {
	public static final int startID = 1;
	public static final int endID = 1000000000;
	public static final int trials = 50;
	public static final boolean DEBUG_MODE = false;

	//public static final String TABLE_ ="_";

	private Connection connection = null;

	public SQLHelper(Connection connection) {
		this.connection = connection;
	}

	public Connection getConnection() {
		return this.connection;
	}

	/**
	 * Insert the method's description here.
	 * Creation date: (19/02/2001 14:52:15)
	 * @return java.lang.Object[]
	 */
	public Object[] getColumnFromTable(String columnName, String tableName) {
		return getColumnFromTable(columnName, tableName, "1=1");
		//Condition is always true
	}

	/**
	 * Insert the method's description here.
	 * Creation date: (19/02/2001 14:52:15)
	 * @return java.lang.Object[]
	 */
	public Object[] getColumnFromTable(
		String columnName,
		String tableName,
		String searchCondition) {
		Connection selectConnection = null;
		Statement stmt = null;
		ResultSet rs = null;
		Vector tempVec = new Vector();
		Object[] ret = new Object[0]; // We don't want to return null

		try {
			selectConnection = getConnection();
			StringBuffer querryStringBuffer = new StringBuffer(50);
			querryStringBuffer.append("SELECT ");
			querryStringBuffer.append(columnName);
			querryStringBuffer.append(" FROM ");
			querryStringBuffer.append(tableName);
			querryStringBuffer.append(" WHERE ");
			querryStringBuffer.append(searchCondition);

			//"SELECT "+columnName+" FROM "+TableName+" WHERE "+searchCondition
			stmt = selectConnection.createStatement();
			rs = stmt.executeQuery(querryStringBuffer.toString());

			Object tmp;
			ResultSetMetaData rsMeta = rs.getMetaData();
			String typeName = rsMeta.getColumnTypeName(1);
			while (rs.next()) {
				if (typeName.equals("CLOB"))
					tmp = rs.getString(1);
				else
					tmp = rs.getObject(1);
				tempVec.add(tmp);
			}

			if (tempVec.isEmpty())
				return ret; //No rows in ResultSet
			ret = new Object[tempVec.size()];
			tempVec.copyInto(ret);

		} catch (Exception e) {
			e.printStackTrace();
			return ret; //No rows in ResultSet
		} finally {
			try {
				if (rs != null)
					rs.close();
				if (stmt != null)
					stmt.close();
			} catch (Exception e) {
			}
			try {
				if (selectConnection != null)
					selectConnection.close();
			} catch (Exception e) {
			}
		}

		return ret;
	}

	public Object[][] getColumnsFromTable(
		String[] columnNames,
		String tableName,
		String searchCondition) {
		String[] tableNames = new String[] { tableName };
		return getColumnsFromTables(columnNames, tableNames, searchCondition);
	}

	public Object[][] getColumnsFromTables(
		String[] columnNames,
		String[] tableNames,
		String searchCondition) {
		Connection selectConnection = null;
		Statement stmt = null;
		ResultSet rs = null;
		Vector tempVec = new Vector();
		Object[][] ret = new Object[0][]; // We don't want to return null
		
		try {
			selectConnection = getConnection();
			stmt = selectConnection.createStatement();
			String queryString = "SELECT ";
			for (int i = 0; i < columnNames.length; i++) {
				queryString += columnNames[i];
				if (i < columnNames.length - 1)
					queryString += ", ";
			}
			queryString += " FROM ";
			for (int i = 0; i < tableNames.length; i++) {
				queryString += tableNames[i];
				if (i < tableNames.length - 1)
					queryString += ", ";
			}

			queryString += " WHERE " + searchCondition;
		
			rs = stmt.executeQuery(queryString);
			//System.out.println(queryString);
			
			//System.err.println("Go to hell!: "+queryString);
			
			ResultSetMetaData rsMeta = rs.getMetaData();
			while (rs.next()) {
				Object[] temp = new Object[columnNames.length];
				for (int i = 0; i < columnNames.length; i++)
					if (rsMeta.getColumnTypeName(i + 1).equals("CLOB"))
						temp[i] = rs.getString(i + 1);
					else
						temp[i] = rs.getObject(i + 1);
				tempVec.add(temp);
			}

			if (tempVec.isEmpty())
				return ret; //No rows in ResultSet
			ret = new Object[tempVec.size()][];
			tempVec.copyInto(ret);

		} catch (Exception e) {
		
			e.printStackTrace();
			return ret; //No rows in ResultSet
		} finally {
			try {
				if (rs != null)
					rs.close();
				if (stmt != null)
					stmt.close();
			} catch (Exception e) {
			}
			try {
				if (selectConnection != null)
					selectConnection.close();
			} catch (Exception e) {
			}
		}

		return ret;
	}

	public boolean setColumnValue(
		String columnName,
		String dbTableName,
		Object value,
		String searchString) {
		Connection updateConnection = null;
		Statement stmt = null;
		String finalValue;

		if (value == null)
			finalValue = "NULL";
		else
			if (value instanceof Number)
				finalValue = value.toString();
			else
				finalValue = "'" + SQLNormalize(value.toString()) + "'";
		//Usually this means that value is a String

		StringBuffer querryStringBuffer = new StringBuffer(60);
		querryStringBuffer.append("UPDATE ");
		querryStringBuffer.append(dbTableName);
		querryStringBuffer.append(" SET ");
		querryStringBuffer.append(columnName);
		querryStringBuffer.append(" = ");
		querryStringBuffer.append(finalValue);
		querryStringBuffer.append(" WHERE ");
		querryStringBuffer.append(searchString);

		try {
			updateConnection = getConnection();
			stmt = updateConnection.createStatement();
			stmt.executeUpdate(querryStringBuffer.toString());
			//			stmt.executeUpdate("UPDATE "+dbTableName+" SET "+columnName+" = "+finalValue+"  WHERE "+searchString);
		} catch (Exception e) {
			e.printStackTrace();
			return false;
		} finally {
			try {
				if (stmt != null)
					stmt.close();
			} catch (Exception e) {
			}
			try {
				if (updateConnection != null)
					updateConnection.close();
			} catch (Exception e) {
			}
		}
		return true;
	}
	public ResultSet executeQuery(String statementText) {
		return executeQuery(statementText, false);
	}

	public ResultSet executeQuery(String statementText, boolean keepOpen) {
		Statement statement = null;
		ResultSet resultSet = null;
		Connection conn = getConnection();
		try {
			statement = conn.createStatement();
			resultSet = statement.executeQuery(statementText);
		} catch (Throwable e) {
			e.printStackTrace();
			return null;
		} finally {
			if (!keepOpen) {
				try {
					if (resultSet != null)
						resultSet.close();
					if (statement != null)
						statement.close();
				} catch (Exception e) {
				}
				try {
					if (conn != null)
						conn.close();
				} catch (Exception e) {
				}
			}
		}
		return resultSet;
	}

	public static String SQLNormalize(String s) {
		StringBuffer str = new StringBuffer();
		if ((s == null) || (s == ""))
			return s;
		else {
			int len = (s != null) ? s.length() : 0;
			for (int i = 0; i < len; i++) {
				char ch = s.charAt(i);
				switch (ch) {
					case '\'' :
						{
							str.append("''");
							break;
						}
					default :
						{
							str.append(ch);
						}
				}
			}

			return str.toString();
		}
	} // normalize(String):String

}